@@ -8,12 +8,14 @@ require 'test/unit'
 require 'senna'
 require 'test_tools'
 
-$KCODE = 'u'
+$KCODE = 'e'
 
 class FixedBugRev00644Test < Test::Unit::TestCase
+  include TestTools
+
   def test_many_spaces_snippet
-    res = Senna::Snip::open('夢', 300, 1, '(', ')', nil, Senna::ENC_UTF8).
-    exec('スペースいっぱい　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　夢いっぱい。')
-    assert_equal(['(　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　　夢)い'], res)
+    res = Senna::Snip::open(euc2utf8('̴'), 300, 1, '(', ')', nil, Senna::ENC_UTF8).
+    exec(euc2utf8('ڡäѤ̴äѤ'))
+    assert_equal([euc2utf8('(̴)')], res)
   end
 end
