@@ -6,8 +6,22 @@
 # the GNU General Public License version 2.
 require 'senna'
 require 'tmpdir'
+require 'nkf'
+
+$KCODE = 'e'
 
 module TestTools
+  def euc2sjis(str)
+    NKF.nkf('-Esx', str)
+  end
+
+  def euc2utf8(str)
+    NKF.nkf('-Ewx', str)
+  end
+
+  def utf82euc(str)
+    NKF.nkf('-Wex', str)
+  end
 end
 
 class TestIndex < Senna::Index
