@@ -4,22 +4,16 @@
 #
 # You can redistribute it and/or modify it under the terms of
 # the GNU General Public License version 2.
-require 'test/unit'
-require 'senna'
-require 'test_tools'
-require 'nkf'
 
 $KCODE = 'e'
 
-def utf8(str)
-  NKF.nkf('-Ew', str)
-end
-
-def euc(str)
-  NKF.nkf('-eW', str)
-end
+require 'test/unit'
+require 'senna'
+require 'test_tools'
 
 class SnippetUtilityTest < Test::Unit::TestCase
+  include TestTools
+
   def test_open
     snip = Senna::Snip::open(nil, 80, 3, '', '', nil, Senna::ENC_DEFAULT)
     assert_equal(Senna::RC_SUCCESS, snip.close)
@@ -207,11 +201,11 @@ class SnippetUtilityTest < Test::Unit::TestCase
     # encodings
     
     # check normalize
-    res = Senna::Snip::open(utf8(''), 6, 1, '.', '.', nil, Senna::ENC_UTF8).exec(utf8('ơ'))
-    assert_equal('.ơ.', euc(res[0]))
+    res = Senna::Snip::open(euc2utf8(''), 6, 1, '.', '.', nil, Senna::ENC_UTF8).exec(euc2utf8('ơ'))
+    assert_equal('.ơ.', utf82euc(res[0]))
 
-    res = Senna::Snip::open(utf8('ơ'), 3, 1, '.', '.', nil, Senna::ENC_UTF8).exec(utf8(''))
-    assert_equal('..', euc(res[0]))
+    res = Senna::Snip::open(euc2utf8('ơ'), 3, 1, '.', '.', nil, Senna::ENC_UTF8).exec(euc2utf8(''))
+    assert_equal('..', utf82euc(res[0]))
     
     # checks nested tag
     res = Senna::Snip::open( { '' => ['<>', '</>'],
@@ -229,27 +223,43 @@ class SnippetUtilityTest < Test::Unit::TestCase
 
     # checks nested tag with overlap and normalize
     snip = Senna::Snip::open( nil, 15, 1, '', '', nil, Senna::ENC_UTF8)
-    snip.add(utf8('ߥ'), utf8('<ߥ>'), utf8('</ߥ>'))
-    snip.add(utf8('С'), utf8('<С>'), utf8('</С>'))
-    snip.add(utf8('С'), utf8('<С>'), utf8('</С>'))
-    res = snip.exec(utf8(''))
+    snip.add(euc2utf8('ߥ'), euc2utf8('<ߥ>'), euc2utf8('</ߥ>'))
+    snip.add(euc2utf8('С'), euc2utf8('<С>'), euc2utf8('</С>'))
+    snip.add(euc2utf8('С'), euc2utf8('<С>'), euc2utf8('</С>'))
+    res = snip.exec(euc2utf8(''))
     assert_equal(1, res.size)
-    assert_equal('<ߥ></ߥ>', euc(res[0]))
+    assert_equal('<ߥ></ߥ>', utf82euc(res[0]))
 
     res = Senna::Snip::open( { '.' => ['<.>', '</.>'] },
-                             3, 1, '', '', nil, Senna::ENC_UTF8).exec(utf8(''))
+                             3, 1, '', '', nil, Senna::ENC_UTF8).exec(euc2utf8(''))
     assert_equal(1, res.size)
-    assert_equal('<.></.>', euc(res[0]))
+    assert_equal('<.></.>', utf82euc(res[0]))
 
     snip = Senna::Snip::open(nil, 18, 1, '', '', nil, Senna::ENC_UTF8)
-    snip.add(utf8('᡼'), utf8('<᡼>'), utf8('</᡼>'))
-    snip.add(utf8('᡼'), utf8('<᡼>'), utf8('</᡼>'))
-    snip.add(utf8('륻'), utf8('<륻>'), utf8('</륻>'))
-    snip.add(utf8('ȥ'), utf8('<ȥ>'), utf8('</ȥ>'))
-    res = snip.exec(utf8('í­ŭ­'))
+    snip.add(euc2utf8('᡼'), euc2utf8('<᡼>'), euc2utf8('</᡼>'))
+    snip.add(euc2utf8('᡼'), euc2utf8('<᡼>'), euc2utf8('</᡼>'))
+    snip.add(euc2utf8('륻'), euc2utf8('<륻>'), euc2utf8('</륻>'))
+    snip.add(euc2utf8('ȥ'), euc2utf8('<ȥ>'), euc2utf8('</ȥ>'))
+    res = snip.exec(euc2utf8('í­ŭ­'))
     assert_equal(1, res.size)
-    assert_equal('<᡼></᡼><ȥ>­ŭ</ȥ>', euc(res[0]))
+    assert_equal('<᡼></᡼><ȥ>­ŭ</ȥ>', utf82euc(res[0]))
 
   end
+
+  def test_remove_leading_space
+    res = Senna::Snip::open({ 'test' => ['<test>', '</test>'],
+                              'doc' => ['<doc>', '</doc>'] },
+                            10, 1, '', '', nil, nil, Senna::SNIP_SKIP_LEADING_SPACES).exec(
+                            'this is test document')
+    assert_equal(1, res.size)
+    assert_equal([' <test>test</test> <doc>doc</doc>u'], res)
+
+    res = Senna::Snip::open({ 'test' => ['<test>', '</test>'],
+                              'doc' => ['<doc>', '</doc>'] },
+                            10, 1, '', '', nil, nil, Senna::SNIP_SKIP_LEADING_SPACES).exec(
+                            'this is                                                   test                document')
+    assert_equal(1, res.size)
+    assert_equal(['   <test>test</test>   '], res)
+  end
 end
 
