@@ -4,12 +4,12 @@
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
-  
+
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
-  
+
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -17,8 +17,8 @@
 
 #ifdef USE_AIO
 
-#ifndef __CACHE_H__
-#define __CACHE_H__
+#ifndef SEN_CACHE_H
+#define SEN_CACHE_H
 
 /* feature switch */
 extern int sen_aio_enabled;
@@ -97,6 +97,6 @@ dp(char *fmt, ...)
   }
 }
 
-#endif /* _cache_h_ */
+#endif /* SEN_CACHE_H */
 
 #endif /* USE_AIO */
