@@ -222,18 +222,26 @@ sen_rc
 sen_com_event_del(sen_com_event *ev, sen_sock fd)
 {
   if (!ev) { return sen_invalid_argument; }
-#ifdef USE_EPOLL
   {
-    struct epoll_event e;
-    memset(&e, 0, sizeof(struct epoll_event));
-    e.data.fd = fd;
-    if (epoll_ctl(ev->epfd, EPOLL_CTL_DEL, fd, &e) == -1) {
-      LOG_SOCKERR("epoll_ctl");
-      return sen_external_error;
+    sen_com *c;
+    sen_set_eh *eh = sen_set_at(ev->set, &fd, (void **)&c);
+    if (eh) {
+#ifdef USE_EPOLL
+      struct epoll_event e;
+      memset(&e, 0, sizeof(struct epoll_event));
+      e.data.fd = fd;
+      e.events = c->events;
+      if (epoll_ctl(ev->epfd, EPOLL_CTL_DEL, fd, &e) == -1) {
+        LOG_SOCKERR("epoll_ctl");
+        return sen_external_error;
+      }
+#endif /* USE_EPOLL*/
+      return sen_set_del(ev->set, eh);
+    } else {
+      SEN_LOG(sen_log_error, "%04x| fd(%d) not found in ev(%p)", getpid(), fd, ev);
+      return sen_internal_error;
     }
   }
-#endif /* USE_EPOLL*/
-  return sen_set_del(ev->set, sen_set_at(ev->set, &fd, NULL));
 }
 
 sen_rc
@@ -302,8 +310,14 @@ sen_com_event_poll(sen_com_event *ev, int timeout)
     efd = ep->data.fd;
     nevents--;
     if (!sen_set_at(ev->set, &efd, (void *)&com)) {
+      struct epoll_event e;
       SEN_LOG(sen_log_error, "fd(%d) not found in ev->set", efd);
-      return sen_internal_error;
+      memset(&e, 0, sizeof(struct epoll_event));
+      e.data.fd = efd;
+      e.events = ep->events;
+      if (epoll_ctl(ev->epfd, EPOLL_CTL_DEL, efd, &e) == -1) { LOG_SOCKERR("epoll_ctl"); }
+      if (sen_sock_close(efd) == -1) { LOG_SOCKERR("close"); }
+      continue;
     }
     if ((ep->events & SEN_COM_POLLIN)) { com->ev_in(ev, com); }
     if ((ep->events & SEN_COM_POLLOUT)) { com->ev_out(ev, com); }
@@ -313,7 +327,8 @@ sen_com_event_poll(sen_com_event *ev, int timeout)
     nevents--;
     if (!sen_set_at(ev->set, &efd, (void *)&com)) {
       SEN_LOG(sen_log_error, "fd(%d) not found in ev->set", efd);
-      return sen_internal_error;
+      if (sen_sock_close(efd) == -1) { LOG_SOCKERR("close"); }
+      continue;
     }
     if ((ep->revents & SEN_COM_POLLIN)) { com->ev_in(ev, com); }
     if ((ep->revents & SEN_COM_POLLOUT)) { com->ev_out(ev, com); }
