@@ -45,7 +45,7 @@ sen_array_fin(sen_array *a)
 {
   int i;
   sen_ctx *ctx = a->ctx;
-  for (i = 0; i < 8; i++) {
+  for (i = 0; i < SEN_ARRAY_N; i++) {
     void **e = &a->elements[i];
     if (*e) {
       SEN_FREE(*e);
@@ -67,8 +67,8 @@ sen_array_id(sen_array *a, void *p)
 {
   uint32_t i, s, n;
   uintptr_t o, p_ = (uintptr_t) p;
-  for (i = 0, s = 0; i < 8; i++, s += n) {
-    n = 0x0f * (1 << (i << 2));
+  for (i = 0, s = 0; i < SEN_ARRAY_N; i++, s += n) {
+    n = SEN_ARRAY_S * SEN_ARRAY_R(i);
     o = (uintptr_t) a->elements[i];
     if (o <= p_ && p_ < o + n * a->element_size) {
       return s + ((p_ - o) / a->element_size);
@@ -173,7 +173,7 @@ sen_str2timeval(const char *str, uint32_t str_len, sen_timeval *tv)
 }
 
 #ifdef USE_FAIL_MALLOC
-double sen_fmalloc_prob = 1.0;
+int sen_fmalloc_prob = 0;
 char *sen_fmalloc_func = NULL;
 char *sen_fmalloc_file = NULL;
 int sen_fmalloc_line = 0;
@@ -337,7 +337,7 @@ sen_init(void)
 #endif /* USE_AIO */
 #ifdef USE_FAIL_MALLOC
   if (getenv("SEN_FMALLOC_PROB")) {
-    sen_fmalloc_prob = strtod(getenv("SEN_FMALLOC_PROB"), 0);
+    sen_fmalloc_prob = strtod(getenv("SEN_FMALLOC_PROB"), 0) * RAND_MAX;
     if (getenv("SEN_FMALLOC_SEED")) {
       srand((unsigned int)atoi(getenv("SEN_FMALLOC_SEED")));
     } else {
@@ -690,7 +690,7 @@ fail_malloc_check(size_t size, const char *file, int line, const char *func)
       (sen_fmalloc_func && strcmp(func, sen_fmalloc_func))) {
     return 1;
   }
-  if (sen_fmalloc_prob * RAND_MAX >= rand()) {
+  if (sen_fmalloc_prob && sen_fmalloc_prob >= rand()) {
     return 0;
   }
   return 1;
