@@ -29,6 +29,10 @@ struct _sen_inv {
   sen_io *chunk;
   sen_sym *lexicon;
   struct sen_inv_header *header;
+#ifdef USE_QUERY_ABORT
+  int (*check_abort)(void*);
+  void *check_abort_arg;
+#endif /* USE_QUERY_ABORT */
   uint32_t total_chunk_size;
   uint16_t ainfo[SEN_INV_MAX_SEGMENT];
   uint16_t binfo[SEN_INV_MAX_SEGMENT];
@@ -1066,6 +1070,10 @@ sen_inv_create08(const char *path, sen_sym *lexicon, uint32_t initial_n_segments
   inv->chunk = chunk;
   inv->header = header;
   inv->lexicon = lexicon;
+#ifdef USE_QUERY_ABORT
+  inv->check_abort = NULL;
+  inv->check_abort_arg = NULL;
+#endif /* USE_QUERY_ABORT */
   inv->total_chunk_size = 0;
   load_all_segments(inv);
   return inv;
