@@ -19,7 +19,9 @@
 
 #include <stdio.h>
 
+#ifndef __USE_GNU
 #define __USE_GNU /* O_DIRECT */
+#endif /* __USE_GNU */
 #include <fcntl.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -97,6 +99,50 @@ inline static int sen_msync(void *start, size_t length);
 inline static sen_rc sen_pread(fileinfo *fi, void *buf, size_t count, off_t offset);
 inline static sen_rc sen_pwrite(fileinfo *fi, void *buf, size_t count, off_t offset);
 
+#define V08      1
+#define TEMPORAL 2
+#define V08P(io) ((io)->flags & V08)
+
+sen_io *
+sen_io_create_tmp(uint32_t header_size, uint32_t segment_size,
+                  uint32_t max_segment, sen_io_mode mode, uint32_t cache_size)
+{
+  sen_io *io;
+  unsigned int b;
+  uint32_t total_header_size;
+  io_header *header;
+  if (!pagesize) { pagesize = sen_get_pagesize(); }
+  total_header_size = IO_HEADER_SIZE + max_segment * sizeof(uint32_t) + header_size;
+  for (b = pagesize; b < total_header_size; b += pagesize);
+  if ((header = (io_header *)SEN_MMAP(NULL, NULL, 0, b))) {
+    header->header_size = header_size;
+    header->segment_size = segment_size;
+    header->max_segment = max_segment;
+    memcpy(header->idstr, SEN_IO_IDSTR, 16);
+    if ((io = SEN_GMALLOCN(sen_io, 1))) {
+      if ((io->maps = SEN_GMALLOCN(sen_io_mapinfo, max_segment))) {
+        memset(io->maps, 0, sizeof(sen_io_mapinfo) * max_segment);
+        io->header = header;
+        io->nrefs = (uint32_t *)(((byte *) header) + IO_HEADER_SIZE);
+        io->user_header = ((byte *) io->nrefs) + max_segment * sizeof(uint32_t);
+        io->base = 0;
+        io->base_seg = 0;
+        io->mode = mode;
+        io->cache_size = cache_size;
+        io->header->curr_size = b;
+        io->fis = NULL;
+        io->nmaps = 0;
+        io->count = 0;
+        io->flags = TEMPORAL;
+        return io;
+      }
+      SEN_GFREE(io);
+    }
+    SEN_MUNMAP(NULL, header, b);
+  }
+  return NULL;
+}
+
 sen_io *
 sen_io_create(const char *path, uint32_t header_size, uint32_t segment_size,
               uint32_t max_segment, sen_io_mode mode, uint32_t cache_size)
@@ -106,7 +152,10 @@ sen_io_create(const char *path, uint32_t header_size, uint32_t segment_size,
   unsigned int b, max_nfiles;
   uint32_t bs, total_header_size;
   io_header *header;
-  if (!path || !*path || (strlen(path) > PATH_MAX - 4)) { return NULL; }
+  if (!path) {
+    return sen_io_create_tmp(header_size, segment_size, max_segment, mode, cache_size);
+  }
+  if (!*path || (strlen(path) > PATH_MAX - 4)) { return NULL; }
   if (!pagesize) { pagesize = sen_get_pagesize(); }
   total_header_size = IO_HEADER_SIZE + max_segment * sizeof(uint32_t) + header_size;
   for (b = pagesize; b < total_header_size; b += pagesize);
@@ -138,7 +187,7 @@ sen_io_create(const char *path, uint32_t header_size, uint32_t segment_size,
             io->fis = fis;
             io->nmaps = 0;
             io->count = 0;
-            io->v08p = 0;
+            io->flags = 0;
             return io;
           }
           SEN_GFREE(io);
@@ -215,7 +264,7 @@ sen_io_open(const char *path, sen_io_mode mode, uint32_t cache_size)
             io->fis = fis;
             io->nmaps = 0;
             io->count = 0;
-            io->v08p = v08p;
+            io->flags = V08;
             return io;
           }
           SEN_GFREE(io->maps);
@@ -249,8 +298,10 @@ sen_io_close(sen_io *io)
     }
   }
   SEN_MUNMAP(&io->fis->fmo, io->header, io->base);
-  for (fi = io->fis, i = max_nfiles; i; fi++, i--) { sen_close(fi); }
-  SEN_GFREE(io->fis);
+  if (io->fis) {
+    for (fi = io->fis, i = max_nfiles; i; fi++, i--) { sen_close(fi); }
+    SEN_GFREE(io->fis);
+  }
   SEN_GFREE(io->maps);
   SEN_GFREE(io);
   return sen_success;
@@ -369,13 +420,193 @@ sen_io_rename(const char *old_name, const char *new_name)
   return sen_file_operation_error;
 }
 
+typedef struct {
+  sen_io_ja_ehead head;
+  char body[256];
+} ja_element;
+
+sen_rc
+sen_io_read_ja(sen_io *io, sen_ctx *ctx, sen_io_ja_einfo *einfo, uint32_t epos, uint32_t key,
+               uint32_t segment, uint32_t offset, void **value, uint32_t *value_len)
+{
+  uint32_t rest = 0, size = *value_len + sizeof(sen_io_ja_ehead);
+  uint32_t segment_size = io->header->segment_size;
+  uint32_t segments_per_file = SEN_IO_FILE_SIZE / segment_size;
+  uint32_t bseg = segment + io->base_seg;
+  int fno = bseg / segments_per_file;
+  fileinfo *fi = &io->fis[fno];
+  off_t base = fno ? 0 : io->base - (uint64_t)segment_size * io->base_seg;
+  off_t pos = (uint64_t)segment_size * (bseg % segments_per_file) + offset + base;
+  ja_element *v = SEN_MALLOC(size);
+  if (!v) {
+    *value = NULL;
+    *value_len = 0;
+    return sen_memory_exhausted;
+  }
+  if (pos + size > SEN_IO_FILE_SIZE) {
+    rest = pos + size - SEN_IO_FILE_SIZE;
+    size = SEN_IO_FILE_SIZE - pos;
+  }
+  if (!sen_opened(fi)) {
+    char path[PATH_MAX];
+    gen_pathname(io->path, path, fno);
+    if (sen_open(fi, path, O_RDWR|O_CREAT, SEN_IO_FILE_SIZE)) {
+      *value = NULL;
+      *value_len = 0;
+      SEN_FREE(v);
+      return sen_file_operation_error;
+    }
+  }
+  if (sen_pread(fi, v, size, pos)) {
+    *value = NULL;
+    *value_len = 0;
+    SEN_FREE(v);
+    return sen_file_operation_error;
+  }
+  if (einfo->pos != epos) {
+    SEN_LOG(sen_log_warning, "einfo pos changed %x => %x", einfo->pos, epos);
+    *value = NULL;
+    *value_len = 0;
+    SEN_FREE(v);
+    return sen_internal_error;
+  }
+  if (einfo->size != *value_len) {
+    SEN_LOG(sen_log_warning, "einfo size changed %d => %d", einfo->size, *value_len);
+    *value = NULL;
+    *value_len = 0;
+    SEN_FREE(v);
+    return sen_internal_error;
+  }
+  if (v->head.key != key) {
+    SEN_LOG(sen_log_error, "ehead key unmatch %x => %x", key, v->head.key);
+    *value = NULL;
+    *value_len = 0;
+    SEN_FREE(v);
+    return sen_invalid_format;
+  }
+  if (v->head.size != *value_len) {
+    SEN_LOG(sen_log_error, "ehead size unmatch %d => %d", *value_len, v->head.size);
+    *value = NULL;
+    *value_len = 0;
+    SEN_FREE(v);
+    return sen_invalid_format;
+  }
+  if (rest) {
+    byte *vr = (byte *)v + size;
+    do {
+      fi = &io->fis[++fno];
+      if (!sen_opened(fi)) {
+        char path[PATH_MAX];
+        gen_pathname(io->path, path, fno);
+        if (sen_open(fi, path, O_RDWR|O_CREAT, SEN_IO_FILE_SIZE)) {
+          *value = NULL;
+          *value_len = 0;
+          SEN_FREE(v);
+          return sen_file_operation_error;
+        }
+      }
+      size = rest > SEN_IO_FILE_SIZE ? SEN_IO_FILE_SIZE : rest;
+      if (sen_pread(fi, vr, size, 0)) {
+        *value = NULL;
+        *value_len = 0;
+        SEN_FREE(v);
+        return sen_file_operation_error;
+      }
+      vr += size;
+      rest -= size;
+    } while (rest);
+  }
+  *value = v->body;
+  return sen_success;
+}
+
+sen_rc
+sen_io_write_ja(sen_io *io, sen_ctx *ctx, uint32_t key,
+                uint32_t segment, uint32_t offset, void *value, uint32_t value_len)
+{
+  sen_rc rc;
+  uint32_t rest = 0, size = value_len + sizeof(sen_io_ja_ehead);
+  uint32_t segment_size = io->header->segment_size;
+  uint32_t segments_per_file = SEN_IO_FILE_SIZE / segment_size;
+  uint32_t bseg = segment + io->base_seg;
+  int fno = bseg / segments_per_file;
+  fileinfo *fi = &io->fis[fno];
+  off_t base = fno ? 0 : io->base - (uint64_t)segment_size * io->base_seg;
+  off_t pos = (uint64_t)segment_size * (bseg % segments_per_file) + offset + base;
+  if (pos + size > SEN_IO_FILE_SIZE) {
+    rest = pos + size - SEN_IO_FILE_SIZE;
+    size = SEN_IO_FILE_SIZE - pos;
+  }
+  if (!sen_opened(fi)) {
+    char path[PATH_MAX];
+    gen_pathname(io->path, path, fno);
+    if ((rc = sen_open(fi, path, O_RDWR|O_CREAT, SEN_IO_FILE_SIZE))) { return rc; }
+  }
+  if (value_len <= 256) {
+    ja_element je;
+    je.head.size = value_len;
+    je.head.key = key;
+    memcpy(je.body, value, value_len);
+    rc = sen_pwrite(fi, &je, size, pos);
+  } else {
+    sen_io_ja_ehead eh;
+    eh.size = value_len;
+    eh.key =  key;
+    if ((rc = sen_pwrite(fi, &eh, sizeof(sen_io_ja_ehead), pos))) { return rc; }
+    pos += sizeof(sen_io_ja_ehead);
+    rc = sen_pwrite(fi, value, size - sizeof(sen_io_ja_ehead), pos);
+  }
+  if (rc) { return rc; }
+  if (rest) {
+    byte *vr = (byte *)value + size - sizeof(sen_io_ja_ehead);
+    do {
+      fi = &io->fis[++fno];
+      if (!sen_opened(fi)) {
+        char path[PATH_MAX];
+        gen_pathname(io->path, path, fno);
+        if ((rc = sen_open(fi, path, O_RDWR|O_CREAT, SEN_IO_FILE_SIZE))) { return rc; }
+      }
+      size = rest > SEN_IO_FILE_SIZE ? SEN_IO_FILE_SIZE : rest;
+      if ((rc = sen_pwrite(fi, vr, size, 0))) { return rc; }
+      vr += size;
+      rest -= size;
+    } while (rest);
+  }
+  return rc;
+}
+
+sen_rc
+sen_io_write_ja_ehead(sen_io *io, sen_ctx *ctx, uint32_t key,
+                      uint32_t segment, uint32_t offset, uint32_t value_len)
+{
+  sen_rc rc;
+  uint32_t segment_size = io->header->segment_size;
+  uint32_t segments_per_file = SEN_IO_FILE_SIZE / segment_size;
+  uint32_t bseg = segment + io->base_seg;
+  int fno = bseg / segments_per_file;
+  fileinfo *fi = &io->fis[fno];
+  off_t base = fno ? 0 : io->base - (uint64_t)segment_size + io->base_seg;
+  off_t pos = (uint64_t)segment_size * (bseg % segments_per_file) + offset + base;
+  if (!sen_opened(fi)) {
+    char path[PATH_MAX];
+    gen_pathname(io->path, path, fno);
+    if ((rc = sen_open(fi, path, O_RDWR|O_CREAT, SEN_IO_FILE_SIZE))) { return rc; }
+  }
+  {
+    sen_io_ja_ehead eh;
+    eh.size = value_len;
+    eh.key =  key;
+    return sen_pwrite(fi, &eh, sizeof(sen_io_ja_ehead), pos);
+  }
+}
+
 void *
 sen_io_win_map(sen_io *io, sen_ctx *ctx, sen_io_win *iw, uint32_t segment,
                uint32_t offset, uint32_t size, sen_io_rw_mode mode)
 {
   byte *p;
-  off_t pos;
-  int fno, base;
+  off_t pos, base;
+  int fno;
   uint32_t nseg, bseg;
   uint32_t segment_size = io->header->segment_size;
   uint32_t segments_per_file = SEN_IO_FILE_SIZE / segment_size;
@@ -388,8 +619,8 @@ sen_io_win_map(sen_io *io, sen_ctx *ctx, sen_io_win *iw, uint32_t segment,
   nseg = (offset + size + segment_size - 1) / segment_size;
   bseg = segment + io->base_seg;
   fno = bseg / segments_per_file;
-  base = fno ? 0 : io->base - io->base_seg * segment_size;
-  pos = (bseg % segments_per_file) * segment_size + offset + base;
+  base = fno ? 0 : io->base - (uint64_t)segment_size * io->base_seg;
+  pos = (uint64_t)segment_size * (bseg % segments_per_file) + offset + base;
   if (!size || !io || segment + nseg > io->header->max_segment ||
       fno != (bseg + nseg - 1) / segments_per_file) {
     return NULL;
@@ -424,17 +655,17 @@ sen_io_win_map(sen_io *io, sen_ctx *ctx, sen_io_win *iw, uint32_t segment,
           return NULL;
         }
       }
-      if (!(p = SEN_MMAP(&iw->fmo, fi, pos, nseg * segment_size))) {
+      if (!(p = SEN_MMAP(&iw->fmo, fi, pos, (uint64_t)segment_size * nseg))) {
         return NULL;
       }
-      if (!io->v08p) {
-        uint64_t tail = io->base + segment * segment_size + offset + size;
+      if (!V08P(io)) {
+        uint64_t tail = io->base + (uint64_t)segment_size * segment + offset + size;
         if (tail > io->header->curr_size) { io->header->curr_size = tail; }
       }
     } else {
       SEN_LOG(sen_log_alert, "nseg == 0! in sen_io_win_map(%p, %u, %u, %u)", io, segment, offset, size);
+      // SEN_IO_SEG_REF(io, segment, p); if (!p) { return NULL; }
       return NULL;
-      // if (!(p = sen_io_seg_ref(io, segment))) { return NULL; }
     }
     iw->addr = p + offset;
     break;
@@ -479,8 +710,8 @@ retry:
         sen_io *io = iw->io;
         uint32_t segment = iw->segment, offset = iw->offset, size = iw->size;
         byte *p;
-        off_t pos;
-        int fno, base;
+        off_t pos, base;
+        int fno;
         uint32_t nseg, bseg;
         uint32_t segment_size = io->header->segment_size;
         uint32_t segments_per_file = SEN_IO_FILE_SIZE / segment_size;
@@ -493,8 +724,8 @@ retry:
         nseg = (offset + size + segment_size - 1) / segment_size;
         bseg = segment + io->base_seg;
         fno = bseg / segments_per_file;
-        base = fno ? 0 : io->base - io->base_seg * segment_size;
-        pos = (bseg % segments_per_file) * segment_size + offset + base;
+        base = fno ? 0 : io->base - (uint64_t)segment_size * io->base_seg;
+        pos = (uint64_t)segment_size * (bseg % segments_per_file) + offset + base;
         if (!size || !io || segment + nseg > io->header->max_segment ||
             fno != (bseg + nseg - 1) / segments_per_file) {
           return sen_abnormal_error;
@@ -638,9 +869,13 @@ sen_io_win_unmap(sen_io_win *iw)
   case sen_io_rdwr:
     // if (nseg > 1) { /* auto unmap is not implemented yet */
     if (nseg > 0) {
-      SEN_MUNMAP(&iw->fmo, ((byte *)iw->addr) - iw->offset, nseg * segment_size);
+      SEN_MUNMAP(&iw->fmo, ((byte *)iw->addr) - iw->offset, (uint64_t)segment_size * nseg);
     } else {
-      rc = sen_io_seg_unref(io, iw->segment);
+      if (iw->segment >= io->header->max_segment) {
+        rc = sen_invalid_argument;
+      } else {
+        SEN_IO_SEG_UNREF(io, iw->segment);
+      }
 #ifdef USE_AIO
       if (sen_aio_enabled) {
         int fno = (iw->segment + io->base_seg) / segments_per_file;
@@ -662,8 +897,8 @@ sen_io_win_unmap(sen_io_win *iw)
       }
       if (!rc) {
         if (!(rc = sen_pwrite(fi, iw->addr, iw->size, iw->pos))) {
-          if (!io->v08p) {
-            uint64_t tail = io->base + iw->segment * segment_size + iw->offset + iw->size;
+          if (!V08P(io)) {
+            uint64_t tail = io->base + (uint64_t)segment_size * iw->segment + iw->offset + iw->size;
             if (tail > io->header->curr_size) { io->header->curr_size = tail; }
           }
           if (!iw->cached) { SEN_FREE(iw->addr); }
@@ -683,36 +918,38 @@ sen_io_win_unmap(sen_io_win *iw)
   return rc;
 }
 
+#define DO_MAP(io,fmo,fi,pos,size,segno,res)\
+{\
+  if (((res) = SEN_MMAP((fmo), (fi), (pos), (size)))) {\
+    uint32_t nmaps;\
+    SEN_ATOMIC_ADD_EX(&io->nmaps, 1, nmaps);\
+    if (!V08P(io)) {\
+      uint64_t tail = io->base + (uint64_t)(size) * ((segno) + 1);\
+      if (tail > io->header->curr_size) { io->header->curr_size = tail; }\
+    }\
+  }\
+}
+
 #define SEG_MAP(io,segno,info)\
 {\
   uint32_t segment_size = io->header->segment_size;\
-  uint32_t segments_per_file = SEN_IO_FILE_SIZE / segment_size;\
-  uint32_t bseg = segno + io->base_seg;\
-  uint32_t fno = bseg / segments_per_file;\
-  uint32_t base = fno ? 0 : io->base - io->base_seg * segment_size;\
-  off_t pos = (bseg % segments_per_file) * segment_size + base;\
-  fileinfo *fi = &io->fis[fno];\
-  if (!sen_opened(fi)) {\
-    char path[PATH_MAX];\
-    gen_pathname(io->path, path, fno);\
-    if (!sen_open(fi, path, O_RDWR|O_CREAT, SEN_IO_FILE_SIZE)) {\
-      if ((info->map = SEN_MMAP(&info->fmo, fi, pos, segment_size))) {\
-        uint32_t nmaps;\
-        SEN_ATOMIC_ADD_EX(&io->nmaps, 1, nmaps);\
-        if (!io->v08p) {\
-          uint64_t tail = io->base + (segno + 1) * segment_size;\
-          if (tail > io->header->curr_size) { io->header->curr_size = tail; }\
-        }\
-      }\
-    }\
+  if ((io->flags & TEMPORAL)) {\
+    DO_MAP(io, &info->fmo, NULL, 0, segment_size, segno, info->map);\
   } else {\
-    if ((info->map = SEN_MMAP(&info->fmo, fi, pos, segment_size))) {\
-      uint32_t nmaps;\
-      SEN_ATOMIC_ADD_EX(&io->nmaps, 1, nmaps);\
-      if (!io->v08p) {\
-        uint64_t tail = io->base + (segno + 1) * segment_size;\
-        if (tail > io->header->curr_size) { io->header->curr_size = tail; }\
+    uint32_t segments_per_file = SEN_IO_FILE_SIZE / segment_size;\
+    uint32_t bseg = segno + io->base_seg;\
+    uint32_t fno = bseg / segments_per_file;\
+    off_t base = fno ? 0 : io->base - (uint64_t)segment_size * io->base_seg;\
+    off_t pos = (uint64_t)segment_size * (bseg % segments_per_file) + base;\
+    fileinfo *fi = &io->fis[fno];\
+    if (!sen_opened(fi)) {\
+      char path[PATH_MAX];\
+      gen_pathname(io->path, path, fno);\
+      if (!sen_open(fi, path, O_RDWR|O_CREAT, SEN_IO_FILE_SIZE)) {\
+        DO_MAP(io, &info->fmo, fi, pos, segment_size, segno, info->map);\
       }\
+    } else {\
+      DO_MAP(io, &info->fmo, fi, pos, segment_size, segno, info->map);\
     }\
   }\
 }
@@ -723,58 +960,6 @@ sen_io_seg_map_(sen_io *io, uint32_t segno, sen_io_mapinfo *info)
   SEG_MAP(io, segno, info);
 }
 
-void *
-sen_io_seg_ref(sen_io *io, uint32_t segno)
-{
-  uint32_t retry, *pnref;
-  sen_io_mapinfo *info;
-  if (segno >= io->header->max_segment) { return NULL; }
-  info = &io->maps[segno];
-  /* pnref = &io->nrefs[segno]; */
-  pnref = &info->nref;
-  for (retry = 0;; retry++) {
-    uint32_t nref;
-    SEN_ATOMIC_ADD_EX(pnref, 1, nref);
-    if (nref >= SEN_IO_MAX_REF) {
-      SEN_ATOMIC_ADD_EX(pnref, -1, nref);
-      if (retry >= SEN_IO_MAX_RETRY) {
-        SEN_LOG(sen_log_crit, "deadlock detected! in sen_io_seg_ref(%p, %u)", io, segno);
-        return NULL;
-      }
-      usleep(1000);
-      continue;
-    }
-    if (!info->map) {
-      if (nref) {
-        SEN_ATOMIC_ADD_EX(pnref, -1, nref);
-        if (retry >= SEN_IO_MAX_RETRY) {
-          SEN_LOG(sen_log_crit, "deadlock detected!! in sen_io_seg_ref(%p, %u)", io, segno);
-          return NULL;
-        }
-        usleep(1000);
-        continue;
-      } else {
-        SEG_MAP(io, segno, info);
-      }
-    }
-    break;
-  }
-  info->count = io->count++;
-  return info->map;
-}
-
-sen_rc
-sen_io_seg_unref(sen_io *io, uint32_t segno)
-{
-  uint32_t nref;
-  uint32_t *pnref;
-  if (segno >= io->header->max_segment) { return sen_invalid_argument; }
-  pnref = &io->maps[segno].nref;
-  /* pnref = &io->nrefs[segno]; */
-  SEN_ATOMIC_ADD_EX(pnref, -1, nref);
-  return sen_success;
-}
-
 sen_rc
 sen_io_seg_expire(sen_io *io, uint32_t segno, uint32_t nretry)
 {
@@ -859,7 +1044,7 @@ inline static void *
 sen_mmap(HANDLE *fmo, fileinfo *fi, off_t offset, size_t length)
 {
   void *res;
-
+  if (!fi) { SEN_LOG(sen_log_crit, "FIXME! not implemented"); }
   /* MUTEX_LOCK(fi->mutex); */
   /* try to create fmo */
   *fmo = CreateFileMapping(fi->fh, NULL, PAGE_READWRITE, 0, offset + length, NULL);
@@ -970,6 +1155,7 @@ inline static void *
 sen_mmap(fileinfo *fi, off_t offset, size_t length)
 {
   void *res;
+  if (!fi) { SEN_LOG(sen_log_crit, "FIXME! not implemented"); }
   /* file must be exceeded to SEN_IO_FILE_SIZE when FileMappingObject created.
      and, after fmo created, it's not allowed to expand the size of file.
   DWORD tail = (DWORD)(offset + length);
@@ -1150,23 +1336,36 @@ sen_close(fileinfo *fi)
   return sen_success;
 }
 
+#if defined(MAP_ANON) && !defined(MAP_ANONYMOUS)
+#define MAP_ANONYMOUS MAP_ANON
+#endif
+
+#include <sys/mman.h>
+
 inline static void *
 sen_mmap(fileinfo *fi, off_t offset, size_t length)
 {
   void *res;
-  struct stat s;
-  off_t tail = offset + length;
-  if ((fstat(fi->fd, &s) == -1) ||
-      (s.st_size < tail && ftruncate(fi->fd, tail) == -1)) {
-    SEN_LOG(sen_log_alert, "fstat or ftruncate failed %d", fi->fd);
-    return NULL;
+  int fd, flags;
+  if (fi) {
+    struct stat s;
+    off_t tail = offset + length;
+    fd = fi->fd;
+    if ((fstat(fd, &s) == -1) || (s.st_size < tail && ftruncate(fd, tail) == -1)) {
+      SEN_LOG(sen_log_alert, "fstat or ftruncate failed %d", fd);
+      return NULL;
+    }
+    flags = MAP_SHARED;
+  } else {
+    fd = -1;
+    flags = MAP_PRIVATE|MAP_ANONYMOUS;
   }
-  res = mmap(NULL, length, PROT_READ|PROT_WRITE, MAP_SHARED, fi->fd, offset);
+  res = mmap(NULL, length, PROT_READ|PROT_WRITE, flags, fd, offset);
   if (MAP_FAILED == res) {
     sen_index_expire();
-    res = mmap(NULL, length, PROT_READ|PROT_WRITE, MAP_SHARED, fi->fd, offset);
+    res = mmap(NULL, length, PROT_READ|PROT_WRITE, flags, fd, offset);
     if (MAP_FAILED == res) {
-      GMERR("mmap(%zu,%d,%d)=%s <%zu>", length, fi->fd, offset, strerror(errno), mmap_size);
+      GMERR("mmap(%zu,%d,%d)=%s <%zu>", length, fd, offset, strerror(errno), mmap_size);
     }
     return NULL;
   }
