@@ -145,7 +145,10 @@ get_phrase(sen_query *q)
       break;
     }
     len = sen_str_charlen_nonnull(s, q->str_end, q->encoding);
-    if (len == 1) {
+    if (!len) {
+      /* invalid string containing malformed multibyte char */
+      return NULL;
+    } else if (len == 1) {
       if (*s == SEN_QUERY_QUOTER) {
         q->cur = s + 1;
         break;
