@@ -182,6 +182,7 @@ sen_ql_def_native_func(sen_ctx *ctx, const char *name, sen_ql_native_func *func)
   }
 }
 
+/*
 inline static void
 sen_ctx_igc(sen_ctx *ctx)
 {
@@ -200,6 +201,7 @@ sen_ctx_igc(sen_ctx *ctx)
   ctx->lseqno = ctx->seqno;
   ctx->nbinds = 0;
 }
+*/
 
 #define MARKP(p)        ((p)->flags & SEN_OBJ_MARKED)
 #define REFERERP(p)     ((p)->flags & SEN_OBJ_REFERER)
@@ -263,10 +265,99 @@ E6:
   }
 }
 
+#define MARK2P(p)        ((p)->flags & SEN_OBJ_MARK2)
+
+sen_rc
+sen_ql_obj_mark(sen_ctx *ctx, sen_obj *o)
+{
+  sen_obj *t, *q, *p;
+  t = NULL;
+  p = o;
+  if (MARK2P(o)) { return sen_invalid_argument; }
+E2:
+  p->flags |= SEN_OBJ_MARK2;
+  if (!REFERERP(p)) { goto E6; }
+  q = CAR(p);
+  if (q && !MARK2P(q)) {
+    UNSETREFERER(p);
+    CAR(p) = t;
+    t = p;
+    p = q;
+    goto E2;
+  }
+E5:
+  q = CDR(p);
+  if (q && !MARK2P(q)) {
+    CDR(p) = t;
+    t = p;
+    p = q;
+    goto E2;
+  }
+E6:
+  if (!t) { return sen_success; }
+  q = t;
+  if (!REFERERP(q)) {
+    SETREFERER(q);
+    t = CAR(q);
+    CAR(q) = p;
+    p = q;
+    goto E5;
+  } else {
+    t = CDR(q);
+    CDR(q) = p;
+    p = q;
+    goto E6;
+  }
+  return sen_success;
+}
+
+sen_rc
+sen_ql_obj_unmark(sen_ctx *ctx, sen_obj *o)
+{
+  sen_obj *t, *q, *p;
+  t = NULL;
+  p = o;
+  if (!MARK2P(o)) { return sen_invalid_argument; }
+E2:
+  p->flags &= ~SEN_OBJ_MARK2;
+  if (!REFERERP(p)) { goto E6; }
+  q = CAR(p);
+  if (q && MARK2P(q)) {
+    UNSETREFERER(p);
+    CAR(p) = t;
+    t = p;
+    p = q;
+    goto E2;
+  }
+E5:
+  q = CDR(p);
+  if (q && MARK2P(q)) {
+    CDR(p) = t;
+    t = p;
+    p = q;
+    goto E2;
+  }
+E6:
+  if (!t) { return sen_success; }
+  q = t;
+  if (!REFERERP(q)) {
+    SETREFERER(q);
+    t = CAR(q);
+    CAR(q) = p;
+    p = q;
+    goto E5;
+  } else {
+    t = CDR(q);
+    CDR(q) = p;
+    p = q;
+    goto E6;
+  }
+  return sen_success;
+}
+
 inline static sen_rc
 sen_ctx_mgc(sen_ctx *ctx)
 {
-  sen_set_cursor *sc;
   /*
   if (!(sc = sen_set_cursor_open(ctx->symbols))) { return sen_memory_exhausted; }
   {
@@ -285,20 +376,44 @@ sen_ctx_mgc(sen_ctx *ctx)
   obj_mark(ctx, ctx->value);
   obj_mark(ctx, ctx->phs);
 
-  if (!(sc = sen_set_cursor_open(ctx->objects))) { return sen_memory_exhausted; }
+  ctx->n_entries = 0;
+#ifdef USE_SET_AS_OBJECTS
   {
-    sen_obj *o;
-    sen_set_eh *ep;
-    while ((ep = sen_set_cursor_next(sc, NULL, (void **) &o))) {
-      if (o->flags & SEN_OBJ_MARKED) {
-        o->flags &= ~SEN_OBJ_MARKED;
-      } else {
-        sen_obj_clear(ctx, o);
-        sen_set_del(ctx->objects, ep);
+    sen_set_cursor *sc;
+    if (!(sc = sen_set_cursor_open(ctx->objects))) { return sen_memory_exhausted; }
+    {
+      sen_obj *o;
+      sen_set_eh *ep;
+      while ((ep = sen_set_cursor_next(sc, NULL, (void **) &o))) {
+        if (o->flags & (SEN_OBJ_MARKED|SEN_OBJ_MARK2)) {
+          o->flags &= ~SEN_OBJ_MARKED;
+          ctx->n_entries++;
+        }  else {
+          sen_obj_clear(ctx, o);
+          sen_set_del(ctx->objects, ep);
+        }
       }
     }
+    sen_set_cursor_close(sc);
   }
-  sen_set_cursor_close(sc);
+#else /* USE_SET_AS_OBJECTS */
+  {
+    sen_id id;
+    sen_obj *o;
+    SEN_ARRAY_EACH(&ctx->objects, 1, ctx->seqno, id, o, {
+      if (o->flags & SEN_OBJ_FREE) {
+      } else if (o->flags & (SEN_OBJ_MARKED|SEN_OBJ_MARK2)) {
+        o->flags &= ~SEN_OBJ_MARKED;
+        ctx->n_entries++;
+      } else {
+        sen_obj_clear(ctx, o);
+        o->flags |= SEN_OBJ_FREE;
+        CDR(o) = ctx->freelist;
+        ctx->freelist = o;
+      }
+    });
+  }
+#endif /* USE_SET_AS_OBJECTS */
   ctx->lseqno = ctx->seqno;
   ctx->nbinds = 0;
   ctx->nunbinds = 0;
@@ -449,7 +564,8 @@ sen_ql_feed(sen_ctx *ctx, char *str, uint32_t str_size, int mode)
         Eval_Cycle(ctx);
       }
       break;
-    case SEN_QL_QUITTING:
+    case SEN_QL_QUITTING :
+    case SEN_CTX_QUIT :
       return NIL;
     }
     if (ERRP(ctx, SEN_ERROR)) { ctx->stat = SEN_QL_QUITTING; return F; }
@@ -490,15 +606,12 @@ skipline(sen_ctx *ctx)
 #define HASPROP(p)       ((p)->flags & SEN_OBJ_SYMBOL)
 #define SYMPROP(p)       CDR(p)
 #define SYNTAXP(p)       ((p)->type == sen_ql_syntax)
-#define PROCP(p)         ((p)->type == sen_ql_proc)
 #define SYNTAXNAME(p)    SYMNAME(p)
 #define SYNTAXNUM(p)     ((p)->class)
 #define PROCNUM(p)       IVALUE(p)
-#define CLOSUREP(p)      ((p)->type == sen_ql_closure)
-#define MACROP(p)        ((p)->flags & SEN_OBJ_MACRO)
+#define MACROP(p)        ((p)->type == sen_ql_macro)
 #define CLOSURE_CODE(p)  CAR(p)
 #define CLOSURE_ENV(p)   CDR(p)
-#define CONTINUATIONP(p) ((p)->type == sen_ql_continuation)
 #define CONT_DUMP(p)     CDR(p)
 #define PROMISEP(p)      ((p)->flags & SEN_OBJ_PROMISE)
 #define SETPROMISE(p)    (p)->flags |= SEN_OBJ_PROMISE
@@ -747,7 +860,21 @@ readstrexp(sen_ctx *ctx, char **str, unsigned int *size)
       return TOK_ATOM;
     } else if (src[0] == '\\' && src + 1 < ctx->str_end && len == 1) {
       src++;
-      *dest++ = *src++;
+      switch (*src) {
+      case 'n' :
+        *dest++ = '\n';
+        break;
+      case 'r' :
+        *dest++ = '\r';
+        break;
+      case 't' :
+        *dest++ = '\t';
+        break;
+      default :
+        *dest++ = *src;
+        break;
+      }
+      src++;
     } else {
       while (len--) { *dest++ = *src++; }
     }
@@ -852,6 +979,7 @@ sen_obj_inspect(sen_ctx *ctx, sen_obj *obj, sen_rbuf *buf, int flags)
       }
       break;
     case sen_ql_snip :
+    case sen_ql_symsnip :
       SEN_RBUF_PUTS(buf, "#<SNIP>");
       break;
     case sen_ql_records :
@@ -859,9 +987,9 @@ sen_obj_inspect(sen_ctx *ctx, sen_obj *obj, sen_rbuf *buf, int flags)
       break;
     case sen_ql_bulk :
       if (flags & SEN_OBJ_INSPECT_ESC) {
-        sen_rbuf_str_esc(buf, obj->u.b.value, obj->u.b.size, ctx->encoding);
+        sen_rbuf_str_esc(buf, STRVALUE(obj), STRSIZE(obj), ctx->encoding);
       } else {
-        sen_rbuf_write(buf, obj->u.b.value, obj->u.b.size);
+        sen_rbuf_write(buf, STRVALUE(obj), STRSIZE(obj));
       }
       break;
     case sen_ql_int :
@@ -891,12 +1019,11 @@ sen_obj_inspect(sen_ctx *ctx, sen_obj *obj, sen_rbuf *buf, int flags)
       sen_rbuf_itoa(buf, PROCNUM(obj));
       SEN_RBUF_PUTS(buf, ">");
       break;
+    case sen_ql_macro :
+      SEN_RBUF_PUTS(buf, "#<MACRO>");
+      break;
     case sen_ql_closure :
-      if (MACROP(obj)) {
-        SEN_RBUF_PUTS(buf, "#<MACRO>");
-      } else {
-        SEN_RBUF_PUTS(buf, "#<CLOSURE>");
-      }
+      SEN_RBUF_PUTS(buf, "#<CLOSURE>");
       break;
     case sen_ql_continuation :
       SEN_RBUF_PUTS(buf, "#<CONTINUATION>");
@@ -1153,20 +1280,20 @@ inline static cell *
 opexe(sen_ctx *ctx)
 {
   register cell *x, *y;
-  if (ctx->op == OP_T0LVL || ctx->objects->n_entries > ctx->ncells + GC_THRESHOLD) {
+  if (ctx->op == OP_T0LVL || ctx->n_entries > ctx->ncells + GC_THRESHOLD) {
     if (ctx->gc_verbose) {
       sen_rbuf buf;
       sen_rbuf_init(&buf, 0);
       sen_obj_inspect(ctx, ctx->envir, &buf, SEN_OBJ_INSPECT_ESC);
       *buf.curr = '\0';
-      SEN_LOG(sen_log_notice, "mgc > ncells=%d envir=<%s>", ctx->objects->n_entries, buf.head);
+      SEN_LOG(sen_log_notice, "mgc > ncells=%d envir=<%s>", ctx->n_entries, buf.head);
       sen_rbuf_fin(&buf);
     }
     sen_ctx_mgc(ctx);
     if (ctx->gc_verbose) {
-      SEN_LOG(sen_log_notice, "mgc < ncells=%d", ctx->objects->n_entries);
+      SEN_LOG(sen_log_notice, "mgc < ncells=%d", ctx->n_entries);
     }
-    ctx->ncells = ctx->objects->n_entries;
+    ctx->ncells = ctx->n_entries;
   }
   switch (ctx->op) {
   case OP_LOAD:    /* load */
@@ -1279,7 +1406,7 @@ opexe(sen_ctx *ctx)
       ctx->args = NIL;
       s_goto(ctx, OP_EVAL);
     } else {  /* end */
-      ctx->args = non_alloc_rev(NIL, ctx->args);
+      ctx->args = reverse(ctx, ctx->args);
       ctx->code = CAR(ctx->args);
       ctx->args = CDR(ctx->args);
       s_goto(ctx, OP_APPLY);
@@ -1287,6 +1414,7 @@ opexe(sen_ctx *ctx)
 
   case OP_APPLY:    /* apply 'code' to 'args' */
     if (NATIVE_FUNCP(ctx->code)) {
+      ctx->dump = CONS(ctx->code, ctx->dump);
       ctx->co.func = ctx->code->u.o.func;
       s_goto(ctx, OP_NATIVE);
     } else if (PROCP(ctx->code)) {
@@ -1415,7 +1543,7 @@ opexe(sen_ctx *ctx)
       ctx->args = NIL;
       s_goto(ctx, OP_EVAL);
     } else {  /* end */
-      ctx->args = non_alloc_rev(NIL, ctx->args);
+      ctx->args = reverse(ctx, ctx->args);
       ctx->code = CAR(ctx->args);
       ctx->args = CDR(ctx->args);
       s_goto(ctx, OP_LET2);
@@ -1429,7 +1557,7 @@ opexe(sen_ctx *ctx)
     if (SYMBOLP(CAR(ctx->code))) {  /* named let */
       for (x = CADR(ctx->code), ctx->args = NIL; PAIRP(x); x = CDR(x))
         ctx->args = CONS(CAAR(x), ctx->args);
-      x = mk_closure(ctx, CONS(non_alloc_rev(NIL, ctx->args), CDDR(ctx->code)),
+      x = mk_closure(ctx, CONS(reverse(ctx, ctx->args), CDDR(ctx->code)),
                      ctx->envir);
       CAR(ctx->envir) = CONS(CONS(CAR(ctx->code), x), CAR(ctx->envir));
       ctx->code = CDDR(ctx->code);
@@ -1487,7 +1615,7 @@ opexe(sen_ctx *ctx)
       ctx->args = NIL;
       s_goto(ctx, OP_EVAL);
     } else {  /* end */
-      ctx->args = non_alloc_rev(NIL, ctx->args);
+      ctx->args = reverse(ctx, ctx->args);
       ctx->code = CAR(ctx->args);
       ctx->args = CDR(ctx->args);
       s_goto(ctx, OP_LET2REC);
@@ -1590,7 +1718,7 @@ opexe(sen_ctx *ctx)
     s_goto(ctx, OP_EVAL);
 
   case OP_1MACRO:  /* macro */
-    ctx->value->flags |= SEN_OBJ_MACRO;
+    ctx->value->type = sen_ql_macro;
     for (x = CAR(ctx->envir); x != NIL; x = CDR(x))
       if (CAAR(x) == ctx->code)
         break;
@@ -1856,6 +1984,7 @@ opexe(sen_ctx *ctx)
     s_return(ctx, CONS(UNQUOTESP, CONS(ctx->value, NIL)));
 
   case OP_NATIVE:
+    ctx->dump = CDR(ctx->dump);
     s_return(ctx, ctx->value);
   case OP_QQUOTE0:
     ctx->code = list_deep_copy(ctx, ctx->code);
@@ -2078,6 +2207,7 @@ sen_ql_eval(sen_ctx *ctx, sen_obj *code, sen_obj *objs)
     } else {\
       QLERR("can't compare");\
     }\
+    break;\
   default :\
     r = (memcmp(&x->u.tv, &y->u.tv, sizeof(sen_timeval)) op 0);\
   }\
@@ -2397,8 +2527,13 @@ nf_neq(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
       } else {
         QLERR("can't compare");
       }
+      break;
+    case sen_ql_object :
+      r = (y->type == sen_ql_object && x->class == y->class && x->u.o.self == y->u.o.self);
+      break;
     default :
       r = (memcmp(&x->u.tv, &y->u.tv, sizeof(sen_timeval)) == 0);
+      break;
     }
     x = y;
   } while (PAIRP(args) && r);
@@ -2483,7 +2618,7 @@ nf_proc(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
    * (call-with-current-continuation procedure?) ==> #t
    * in R^3 report sec. 6.9
    */
-  s_retbool(PROCP(CAR(args)) || CLOSUREP(CAR(args)) || CONTINUATIONP(CAR(args)));
+  s_retbool(PROCEDUREP(CAR(args)));
 }
 static sen_obj *
 nf_pair(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
@@ -2584,9 +2719,7 @@ nf_get_closure(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
     return F;
   } else if (CLOSUREP(args)) {
     return CONS(LAMBDA, CLOSURE_CODE(ctx->value));
-  } else if (MACROP(args)) {
-    return CONS(LAMBDA, CLOSURE_CODE(ctx->value));
-  } else {
+  }  else {
     return F;
   }
 }
@@ -2707,7 +2840,8 @@ nf_tonumber(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
     }
     break;
   default :
-    QLERR("can't convert into number");
+    v = mk_number(ctx, 0);
+    break;
   }
   return v;
 }
@@ -2789,6 +2923,56 @@ nf_substrb(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
     return o;
   }
 }
+static sen_obj *
+nf_tob32h(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
+{
+  sen_obj *x, *v;
+  if (!PAIRP(args)) { QLERR("list required"); }
+  x = CAR(args);
+  switch (x->type) {
+  case sen_ql_int :
+    {
+      sen_rbuf buf;
+      sen_rbuf_init(&buf, 13);
+      if (sen_rbuf_lltob32h(&buf, IVALUE(x))) {
+        sen_rbuf_fin(&buf);
+        QLERR("lltob32h failed");
+      }
+      SEN_RBUF2OBJ(ctx, &buf, v);
+    }
+    break;
+  case sen_ql_float :
+    {
+      sen_rbuf buf;
+      sen_rbuf_init(&buf, 13);
+      if (sen_rbuf_lltob32h(&buf, (int64_t)FVALUE(x))) {
+        sen_rbuf_fin(&buf);
+        QLERR("lltob32h failed");
+      }
+      SEN_RBUF2OBJ(ctx, &buf, v);
+    }
+    break;
+  default :
+    QLERR("can't convert into int");
+  }
+  return v;
+}
+static sen_obj *
+nf_intern(sen_ctx *ctx, sen_obj *args, sen_ql_co *co)
+{
+  sen_obj *x, *v;
+  if (!PAIRP(args)) { QLERR("list required"); }
+  x = CAR(args);
+  if SYMBOLP(x) { return x; }
+  switch (x->type) {
+  case sen_ql_bulk :
+    v = sen_ql_mk_symbol2(ctx, STRVALUE(x), STRSIZE(x), 0);
+    break;
+  default :
+    QLERR("can't convert into string");
+  }
+  return v;
+}
 
 /* ========== Initialization of internal keywords ========== */
 
@@ -2930,6 +3114,8 @@ init_procs(sen_ctx *ctx)
   sen_ql_def_native_func(ctx, "x->time", nf_totime);
   sen_ql_def_native_func(ctx, "x->number", nf_tonumber);
   sen_ql_def_native_func(ctx, "substrb", nf_substrb);
+  sen_ql_def_native_func(ctx, "x->b32h", nf_tob32h);
+  sen_ql_def_native_func(ctx, "intern", nf_intern);
 }
 
 /* initialize several globals */
