@@ -142,25 +142,27 @@ sen_set_reset(sen_set * set, uint32_t ne)
   for (n = INITIAL_INDEX_SIZE; n <= ne; n *= 2);
   if (!(index = SEN_CALLOC(n * sizeof(entry *)))) { return sen_memory_exhausted; }
   m = n - 1;
-  if (set->key_size) {
-    for (j = set->max_offset + 1, sp = set->index; j; j--, sp++) {
-      e = *sp;
-      if (!e || (e == GARBAGE)) { continue; }
-      for (i = e->key, s = STEP(i); ; i += s) {
-        dp = index + (i & m);
-        if (!*dp) { break; }
+  if (set->index) {
+    if (set->key_size) {
+      for (j = set->max_offset + 1, sp = set->index; j; j--, sp++) {
+        e = *sp;
+        if (!e || (e == GARBAGE)) { continue; }
+        for (i = e->key, s = STEP(i); ; i += s) {
+          dp = index + (i & m);
+          if (!*dp) { break; }
+        }
+        *dp = e;
       }
-      *dp = e;
-    }
-  } else {
-    for (j = set->max_offset + 1, sp = set->index; j; j--, sp++) {
-      e = *sp;
-      if (!e || (e == GARBAGE)) { continue; }
-      for (i = SEN_SET_STRHASH(e), s = STEP(i); ; i += s) {
-        dp = index + (i & m);
-        if (!*dp) { break; }
+    } else {
+      for (j = set->max_offset + 1, sp = set->index; j; j--, sp++) {
+        e = *sp;
+        if (!e || (e == GARBAGE)) { continue; }
+        for (i = SEN_SET_STRHASH(e), s = STEP(i); ; i += s) {
+          dp = index + (i & m);
+          if (!*dp) { break; }
+        }
+        *dp = e;
       }
-      *dp = e;
     }
   }
   {
@@ -168,7 +170,7 @@ sen_set_reset(sen_set * set, uint32_t ne)
     set->index = index;
     set->max_offset = m;
     set->n_garbages = 0;
-    SEN_FREE(i0);
+    if (i0) { SEN_FREE(i0); }
   }
   return sen_success;
 }
@@ -205,11 +207,13 @@ sen_set_int_at(sen_set *set, const uint32_t *key, void **value)
 {
   entry *e, **ep, **index = set->index;
   uint32_t h = *key, i, m = set->max_offset, s = STEP(h);
-  for (i = h; ep = index + (i & m), (e = *ep); i += s) {
-    if (e == GARBAGE) { continue; }
-    if (e->key == h) {
-      if (value) { *value = e->dummy; }
-      return ep;
+  if (index) {
+    for (i = h; ep = index + (i & m), (e = *ep); i += s) {
+      if (e == GARBAGE) { continue; }
+      if (e->key == h) {
+        if (value) { *value = e->dummy; }
+        return ep;
+      }
     }
   }
   return NULL;
@@ -220,11 +224,13 @@ sen_set_str_at(sen_set *set, const char *key, void **value)
 {
   entry *e, **ep, **index = set->index;
   uint32_t h = str_hash((unsigned char *)key), i, m = set->max_offset, s = STEP(h);
-  for (i = h; ep = index + (i & m), (e = *ep); i += s) {
-    if (e == GARBAGE) { continue; }
-    if (SEN_SET_STRHASH(e) == h && !strcmp(key, SEN_SET_STRKEY(e))) {
-      if (value) { *value = SEN_SET_STRVAL(e); }
-      return (sen_set_eh *) ep;
+  if (index) {
+    for (i = h; ep = index + (i & m), (e = *ep); i += s) {
+      if (e == GARBAGE) { continue; }
+      if (SEN_SET_STRHASH(e) == h && !strcmp(key, SEN_SET_STRKEY(e))) {
+        if (value) { *value = SEN_SET_STRVAL(e); }
+        return (sen_set_eh *) ep;
+      }
     }
   }
   return NULL;
@@ -235,11 +241,13 @@ sen_set_bin_at(sen_set *set, const void *key, void **value)
 {
   entry *e, **ep, **index = set->index;
   uint32_t h = bin_hash(key, set->key_size), i, m = set->max_offset, s = STEP(h);
-  for (i = h; ep = index + (i & m), (e = *ep); i += s) {
-    if (e == GARBAGE) { continue; }
-    if (e->key == h && !memcmp(key, e->dummy, set->key_size)) {
-      if (value) { *value = SEN_SET_BINVAL(e, set); }
-      return ep;
+  if (index) {
+    for (i = h; ep = index + (i & m), (e = *ep); i += s) {
+      if (e == GARBAGE) { continue; }
+      if (e->key == h && !memcmp(key, e->dummy, set->key_size)) {
+        if (value) { *value = SEN_SET_BINVAL(e, set); }
+        return ep;
+      }
     }
   }
   return NULL;
@@ -270,6 +278,7 @@ sen_set_int_get(sen_set *set, const uint32_t *key, void **value)
   entry *e, **ep, **np = NULL, **index = set->index;
   uint32_t h = *key, i, m = set->max_offset, s = STEP(h);
   _ncalls++;
+  if (!index) { return NULL; }
   for (i = h; ep = index + (i & m), e = *ep; i += s) {
     if (e == GARBAGE) {
       if (!np) { np = ep; }
@@ -304,6 +313,7 @@ sen_set_str_get(sen_set *set, const char *key, void **value)
   sen_ctx *ctx = set->ctx;
   entry *e, **ep, **np = NULL, **index = set->index;
   uint32_t h = str_hash((unsigned char *)key), i, m = set->max_offset, s = STEP(h);
+  if (!index) { return NULL; }
   for (i = h; ep = index + (i & m), e = *ep; i += s) {
     if (e == GARBAGE) {
       if (!np) { np = ep; }
@@ -337,6 +347,7 @@ sen_set_bin_get(sen_set *set, const void *key, void **value)
 {
   entry *e, **ep, **np = NULL, **index = set->index;
   uint32_t h = bin_hash(key, set->key_size), i, m = set->max_offset, s = STEP(h);
+  if (!index) { return NULL; }
   for (i = h; ep = index + (i & m), e = *ep; i += s) {
     if (e == GARBAGE) {
       if (!np) { np = ep; }
@@ -402,7 +413,7 @@ sen_set_cursor_open(sen_set *set)
 {
   sen_set_cursor *c;
   sen_ctx *ctx = set->ctx;
-  if (!set) { return NULL; }
+  if (!set || !set->index) { return NULL; }
   if (!(c = SEN_MALLOC(sizeof(sen_set_cursor)))) { return NULL; }
   c->set = set;
   c->index = set->index;
@@ -648,7 +659,7 @@ sen_set_sort(sen_set *set, int limit, sen_set_sort_optarg *optarg)
   sen_ctx *ctx = set->ctx;
   entry **res;
   int dir = 1;
-  if (!set) {
+  if (!set || !set->index) {
     SEN_LOG(sen_log_warning, "sen_set_sort: invalid argument !");
     return NULL;
   }
@@ -975,7 +986,7 @@ sen_rset_group(sen_set *s, int limit, sen_group_optarg *optarg)
   byte *key, *ekey, *gkey = NULL;
   int funcp, dir;
   unsigned int rsize;
-  if (!s) { return NULL; }
+  if (!s || !s->index) { return NULL; }
   if (optarg) {
     unit = sen_rec_userdef;
     rsize = optarg->key_size;
