@@ -17,7 +17,6 @@
 #include "senna_in.h"
 #include <string.h>
 #include <stddef.h>
-#include <assert.h>
 #include "snip.h"
 #include "ctx.h"
 
@@ -85,7 +84,7 @@ sen_bm_preBmBc(const unsigned char *x, size_t m, size_t *bmBc)
 
 #define SEN_BM_COMPARE \
   if (object->checks[found]) { \
-    size_t offset = cond->start_offset, found_alpha_head = cond->found_alpha_head; \
+    size_t offset = cond->last_offset, found_alpha_head = cond->found_alpha_head; \
     /* calc real offset */\
     for (i = cond->last_found; i < found; i++) { \
       if (object->checks[i] > 0) { \
@@ -100,11 +99,12 @@ sen_bm_preBmBc(const unsigned char *x, size_t m, size_t *bmBc)
     } else { \
       cond->last_found = found; \
     } \
+    cond->start_offset = cond->last_offset = offset; \
     if (flags & SEN_SNIP_SKIP_LEADING_SPACES) { \
-      while (offset < object->orig_blen && \
-             (i = sen_isspace(object->orig + offset, object->encoding))) { offset += i; } \
+      while (cond->start_offset < object->orig_blen && \
+             (i = sen_isspace(object->orig + cond->start_offset, \
+                              object->encoding))) { cond->start_offset += i; } \
     } \
-    cond->start_offset = offset; \
     for (i = cond->last_found; i < found + m; i++) { \
       if (object->checks[i] > 0) { \
         offset += object->checks[i]; \
@@ -282,6 +282,7 @@ sen_snip_cond_reinit(snip_cond *cond)
 {
   cond->found = 0;
   cond->last_found = 0;
+  cond->last_offset = 0;
   cond->start_offset = 0;
   cond->end_offset = 0;
 
@@ -332,7 +333,10 @@ sen_snip_add_cond(sen_snip *snip,
   if (closetag) {
     if (snip->flags & SEN_SNIP_COPY_TAG) {
       char *t = SEN_MALLOC(closetag_len + 1);
-      if (!t) { return sen_memory_exhausted; }
+      if (!t) {
+        if (opentag) { SEN_FREE((void *)cond->opentag); }
+        return sen_memory_exhausted;
+      }
       memcpy(t, closetag, closetag_len);
       t[closetag_len]= '\0'; /* not required, but for ql use */
       cond->closetag = t;
@@ -611,9 +615,13 @@ sen_snip_exec(sen_snip *snip, const char *string, unsigned int string_len,
       }
       last_last_end_offset = snip_result->end_offset;
 
-      tagged_len +=
-        count_mapped_chars(&string[snip_result->start_offset],
-                           &string[snip_result->end_offset]) + 1;
+      if (snip->mapping == (sen_snip_mapping *) -1) {
+        tagged_len +=
+          count_mapped_chars(&string[snip_result->start_offset],
+                             &string[snip_result->end_offset]) + 1;
+      } else {
+        tagged_len += snip_result->end_offset - snip_result->start_offset + 1;
+      }
 
       *max_tagged_len = MAX(*max_tagged_len, tagged_len);
 
@@ -648,14 +656,15 @@ sen_snip_get_result(sen_snip *snip, const unsigned int index, char *result, unsi
 {
   char *p;
   size_t i, j, k;
-  _snip_result *sres = &snip->snip_result[index];
+  _snip_result *sres;
 
   if (snip->snip_count <= index || !snip->nstr) {
     return sen_invalid_argument;
   }
 
-  assert(snip->snip_count != 0 && snip->tag_count != 0);
+  SEN_ASSERT(snip->snip_count != 0 && snip->tag_count != 0);
 
+  sres = &snip->snip_result[index];
   j = sres->first_tag_result_idx;
   for (p = result, i = sres->start_offset; i < sres->end_offset; i++) {
     for (; j <= sres->last_tag_result_idx && snip->tag_result[j].start_offset == i; j++) {
@@ -719,7 +728,7 @@ sen_snip_get_result(sen_snip *snip, const unsigned int index, char *result, unsi
   *p = '\0';
 
   if(result_len) { *result_len = (unsigned int)(p - result); }
-  assert((unsigned int)(p - result) <= snip->max_tagged_len);
+  SEN_ASSERT((unsigned int)(p - result) <= snip->max_tagged_len);
 
   return sen_success;
 }
