@@ -21,7 +21,9 @@
 #include "str.h"
 #include "set.h"
 
+#ifndef __USE_ISOC99
 #define __USE_ISOC99
+#endif /* __USE_ISOC99 */
 #include <math.h>
 
 static sen_set *prefix = NULL;
@@ -1574,6 +1576,49 @@ sen_str_btoi(char *b)
   return id ^ MASK;
 }
 
+#define I2B32H(i) ("0123456789ABCDEFGHIJKLMNOPQRSTUV"[(i) & 0x1f])
+
+char *
+sen_str_lltob32h(int64_t i, char *p)
+{
+  uint64_t u = (uint64_t)i + 0x8000000000000000ULL;
+  *p++ = I2B32H(u >> 60);
+  *p++ = I2B32H(u >> 55);
+  *p++ = I2B32H(u >> 50);
+  *p++ = I2B32H(u >> 45);
+  *p++ = I2B32H(u >> 40);
+  *p++ = I2B32H(u >> 35);
+  *p++ = I2B32H(u >> 30);
+  *p++ = I2B32H(u >> 25);
+  *p++ = I2B32H(u >> 20);
+  *p++ = I2B32H(u >> 15);
+  *p++ = I2B32H(u >> 10);
+  *p++ = I2B32H(u >> 5);
+  *p++ = I2B32H(u);
+  return p;
+}
+
+char *
+sen_str_ulltob32h(uint64_t i, char *p)
+{
+  char lb = (i >> 59) & 0x10;
+  i += 0x8000000000000000ULL;
+  *p++ = lb + I2B32H(i >> 60);
+  *p++ = I2B32H(i >> 55);
+  *p++ = I2B32H(i >> 50);
+  *p++ = I2B32H(i >> 45);
+  *p++ = I2B32H(i >> 40);
+  *p++ = I2B32H(i >> 35);
+  *p++ = I2B32H(i >> 30);
+  *p++ = I2B32H(i >> 25);
+  *p++ = I2B32H(i >> 20);
+  *p++ = I2B32H(i >> 15);
+  *p++ = I2B32H(i >> 10);
+  *p++ = I2B32H(i >> 5);
+  *p++ = I2B32H(i);
+  return p;
+}
+
 int
 sen_str_tok(char *str, size_t str_len, char delim, char **tokbuf, int buf_size, char **rest)
 {
@@ -1816,6 +1861,19 @@ sen_rbuf_itob(sen_rbuf *buf, sen_id id)
   return rc;
 }
 
+sen_rc
+sen_rbuf_lltob32h(sen_rbuf *buf, int64_t i)
+{
+  size_t len = 13;
+  sen_rc rc = sen_success;
+  if (SEN_RBUF_REST(buf) < len) {
+    if ((rc = sen_rbuf_resize(buf, SEN_RBUF_VSIZE(buf) + len))) { return rc; }
+  }
+  sen_str_lltob32h(i, buf->curr);
+  buf->curr += len;
+  return rc;
+}
+
 void
 sen_rbuf_str_esc(sen_rbuf *buf, const char *s, int len, sen_encoding encoding)
 {
@@ -1827,6 +1885,9 @@ sen_rbuf_str_esc(sen_rbuf *buf, const char *s, int len, sen_encoding encoding)
     if (!(l = sen_str_charlen_nonnull(s, e, encoding))) { break; }
     if (l == 1) {
       switch (*s) {
+      case '\t' :
+        sen_rbuf_write(buf, "\\t", 2);
+        break;
       case '\n' :
         sen_rbuf_write(buf, "\\n", 2);
         break;
