@@ -104,11 +104,15 @@ enum {
 
 /* bit operation */
 
+/*
 inline static uint32_t
 nth_bit(const byte *key, uint32_t n)
 {
   return key[n >> 3] & (0x80 >> (n & 7));
 }
+*/
+
+#define nth_bit(key,n) (((key)[(n)>>3]) & (0x80 >> ((n) & 7)))
 
 /* segment operation */
 
@@ -174,6 +178,20 @@ pat_at(sen_sym *sym, sen_id id)
   return (pat_node *)(((byte *)*p) + ((id & PAT_MASK_IN_A_SEGMENT) * sizeof(pat_node)));
 }
 
+#define PAT_AT(sym,id,n) do {\
+  void **p_;\
+  sen_id id_ = (id);\
+  uint16_t lseg_ = id_ >> W_OF_PAT_IN_A_SEGMENT;\
+  if (id_ > SEN_SYM_MAX_ID) { (n) = NULL; break; }\
+  p_ = &(sym)->pataddrs[lseg_];\
+  if (!*p_) {\
+    uint16_t pseg_ = (sym)->header->patarray[lseg_];\
+    if (pseg_ == SEG_NOT_ASSIGNED) { (n) = NULL; break; }\
+    SEG_LOAD((sym)->io, pseg_, *p_);\
+    if (!*p_) { (n) = NULL; break; }\
+  }\
+  (n) = (pat_node *)(((byte *)*p_) + ((id_ & PAT_MASK_IN_A_SEGMENT) * sizeof(pat_node)));\
+} while(0)
 
 inline static pat_node *
 pat_get(sen_sym *sym, sen_id id)
@@ -370,9 +388,11 @@ delinfo_turn_2(sen_sym *sym, sen_sym_delinfo *di)
   pat_node *ln, *dn;
   // sen_log("delinfo_turn_2> di->d=%d di->ld=%d stat=%d", di->d, DL_LD(di), DL_ST(di));
   if (DL_ST(di) != DL_PHASE1) { return sen_success; }
-  if (!(ln = pat_at(sym, DL_LD(di)))) { return sen_invalid_argument; }
+  PAT_AT(sym, DL_LD(di), ln);
+  if (!ln) { return sen_invalid_argument; }
   if (!(d = di->d)) { return sen_invalid_argument; }
-  if (!(dn = pat_at(sym, d))) { return sen_invalid_argument; }
+  PAT_AT(sym, d, dn);
+  if (!dn) { return sen_invalid_argument; }
   SET_PAT_DEL(ln, 0);
   SET_PAT_DEL(dn, 0);
   {
@@ -383,7 +403,7 @@ delinfo_turn_2(sen_sym *sym, sen_sym_delinfo *di)
     const uint8_t *key = pat_node_get_key(sym, dn);
     if (!key) { return sen_invalid_argument; }
     if (!len) { len = (strlen((char *)key) + 1) * 8; }
-    rn = pat_at(sym, 0);
+    PAT_AT(sym, 0, rn);
     c = -1;
     p0 = &rn->r;
     while ((r = *p0)) {
@@ -391,7 +411,8 @@ delinfo_turn_2(sen_sym *sym, sen_sym_delinfo *di)
         p = p0;
         break;
       }
-      if (!(rn = pat_at(sym, r))) { return sen_invalid_format; }
+      PAT_AT(sym, r, rn);
+      if (!rn) { return sen_invalid_format; }
       if (rn->check <= c || len <= rn->check) { break; }
       c = rn->check;
       p0 = nth_bit((uint8_t *)key, c) ? &rn->r : &rn->l;
@@ -413,7 +434,8 @@ delinfo_turn_2(sen_sym *sym, sen_sym_delinfo *di)
          j = (j + 1) & SEN_SYM_MDELINFOS) {
       ddi = &sym->header->delinfos[j];
       if (DL_ST(ddi) != DL_PHASE1) { continue; }
-      if (!(ln = pat_at(sym, DL_LD(ddi)))) { continue; }
+      PAT_AT(sym, DL_LD(ddi), ln);
+      if (!ln) { continue; }
       if (!(dd = ddi->d)) { continue; }
       if (d == DL_LD(ddi)) {
         SEN_LOG(sen_log_debug, "found!!!, d(%d) become ld of (%d)", d, dd);
@@ -433,7 +455,8 @@ delinfo_turn_3(sen_sym *sym, sen_sym_delinfo *di)
   pat_node *dn;
   uint32_t size;
   if (DL_ST(di) != DL_PHASE2) { return sen_success; }
-  if (!(dn = pat_at(sym, di->d))) { return sen_invalid_argument; }
+  PAT_AT(sym, di->d, dn);
+  if (!dn) { return sen_invalid_argument; }
   if (DL_SH(di)) {
     SET_PAT_IMD(dn, PAT_IMMEDIATE);
     size = 0;
@@ -550,6 +573,7 @@ sen_sym_open(const char *path)
   int i;
   sen_io *io;
   sen_sym *sym;
+  pat_node *r0;
   struct sen_sym_header *header;
   ERRCLR(NULL);
   io = sen_io_open(path, sen_io_auto, 8192);
@@ -576,7 +600,8 @@ sen_sym_open(const char *path)
     sym->pataddrs[i] = NULL;
     sym->sisaddrs[i] = NULL;
   }
-  if (!pat_at(sym, 0)) {
+  PAT_AT(sym, 0, r0);
+  if (!r0) {
     sen_io_close(io);
     SEN_GFREE(sym);
     return NULL;
@@ -613,7 +638,8 @@ _sen_sym_get(sen_sym *sym, const uint8_t *key, uint32_t *new, uint32_t *lkey)
   if (!size) { size = strlen((char *)key) + 1; }
   len = (int)size * 8;
   if (len > SEN_SYM_MAX_KEY_LENGTH) { return SEN_SYM_NIL; }
-  rn0 = pat_at(sym, 0); p0 = &rn0->r;
+  PAT_AT(sym, 0, rn0);
+  p0 = &rn0->r;
   if (*p0) {
     char xor, mask;
     const uint8_t *s, *d;
@@ -622,7 +648,8 @@ _sen_sym_get(sen_sym *sym, const uint8_t *key, uint32_t *new, uint32_t *lkey)
         if (!(s = pat_node_get_key(sym, rn0))) { return 0; }
         break;
       }
-      if (!(rn0 = pat_at(sym, r0))) { return 0; }
+      PAT_AT(sym, r0, rn0);
+      if (!rn0) { return 0; }
       if (c0 < rn0->check && rn0->check < len) {
         c1 = c0; c0 = rn0->check;
         p1 = p0; p0 = nth_bit(key, c0) ? &rn0->r : &rn0->l;
@@ -646,9 +673,11 @@ e    if (r0 && r0 != sen_sym_at(sym, pat_node_get_key(sym, rn0))) {
         if (c > c1) {
           p0 = p1;
         } else {
-          rn0 = pat_at(sym, 0); p0 = &rn0->r;
+          PAT_AT(sym, 0, rn0);
+          p0 = &rn0->r;
           while ((r0 = *p0)) {
-            if (!(rn0 = pat_at(sym, r0))) { return 0; }
+            PAT_AT(sym, r0, rn0);
+            if (!rn0) { return 0; }
             if (c < rn0->check) { break; }
             p0 = nth_bit(key, rn0->check) ? &rn0->r : &rn0->l;
           }
@@ -665,7 +694,8 @@ e    if (r0 && r0 != sen_sym_at(sym, pat_node_get_key(sym, rn0))) {
       if (sym->header->garbages[0]) {
         r = sym->header->garbages[0];
         sym->header->nrecords++;
-        if (!(rn = pat_at(sym, r))) { return 0; }
+        PAT_AT(sym, r, rn);
+        if (!rn) { return 0; }
         sym->header->garbages[0] = rn->l;
       } else {
         if (!(rn = pat_node_new(sym, &r))) { return 0; }
@@ -677,7 +707,8 @@ e    if (r0 && r0 != sen_sym_at(sym, pat_node_get_key(sym, rn0))) {
         uint8_t *keybuf;
         r = sym->header->garbages[size2];
         sym->header->nrecords++;
-        if (!(rn = pat_at(sym, r))) { return 0; }
+        PAT_AT(sym, r, rn);
+        if (!rn) { return 0; }
         sym->header->garbages[size2] = rn->l;
         if (!(keybuf = pat_node_get_key(sym, rn))) { return 0; }
         memcpy(keybuf, key, size);
@@ -761,11 +792,12 @@ sen_sym_at(sen_sym *sym, const void *key)
   int c = -1;
   size_t len;
   if (!sym || !key) { return SEN_SYM_NIL; }
-  len = sym->key_size * 8;
   if (sym->v08p) { return sen_sym_at08(sym, key); }
-  if (!len) { len = (strlen(key) + 1) * 8; }
-  for (r = pat_at(sym, 0)->r; r; r = nth_bit((uint8_t *)key, c) ? rn->r : rn->l) {
-    if (!(rn = pat_at(sym, r))) { break; /* corrupt? */ }
+  len = (sym->key_size ? sym->key_size : strlen(key) + 1) * 8;
+  PAT_AT(sym, 0, rn);
+  for (r = rn->r; r; r = nth_bit((uint8_t *)key, c) ? rn->r : rn->l) {
+    PAT_AT(sym, r, rn);
+    if (!rn) { break; /* corrupt? */ }
     if (len <= rn->check) { break; }
     if (rn->check <= c) {
       const uint8_t *k = pat_node_get_key(sym, rn);
@@ -798,21 +830,25 @@ get_tc(sen_sym *sym, sen_set *h, pat_node *rn, unsigned int len)
   sen_id id;
   pat_node *node;
   id = rn->r;
-  if (id && (node = pat_at(sym, id))) {
-    if (node->check > rn->check) {
-      sen_set_get(h, &id, NULL);
-      get_tc(sym, h, node, len);
-    } else if (node->check < len) {
-      sen_set_get(h, &id, NULL);
+  if (id) {
+    PAT_AT(sym, id, node);
+    if (node) {
+      if (node->check > rn->check) {
+        get_tc(sym, h, node, len);
+      } else {
+        sen_set_get(h, &id, NULL);
+      }
     }
   }
   id = rn->l;
-  if (id && (node = pat_at(sym, id))) {
-    if (node->check > rn->check) {
-      sen_set_get(h, &id, NULL);
-      get_tc(sym, h, node, len);
-    } else if (node->check < len) {
-      sen_set_get(h, &id, NULL);
+  if (id) {
+    PAT_AT(sym, id, node);
+    if (node) {
+      if (node->check > rn->check) {
+        get_tc(sym, h, node, len);
+      } else {
+        sen_set_get(h, &id, NULL);
+      }
     }
   }
 }
@@ -823,10 +859,13 @@ sen_sym_prefix_search_with_set(sen_sym *sym, const void *key, sen_set *h)
   int c = -1;
   const uint8_t *k;
   size_t len = strlen(key) * 8;
-  pat_node *rn = pat_at(sym, 0);
-  sen_id r = rn->r;
+  sen_id r;
+  pat_node *rn;
+  PAT_AT(sym, 0, rn);
+  r = rn->r;
   while (r) {
-    if (!(rn = pat_at(sym, r))) { return sen_invalid_format; }
+    PAT_AT(sym, r, rn);
+    if (!rn) { return sen_invalid_format; }
     if (c < rn->check && len > rn->check) {
       c = rn->check;
       r = nth_bit((uint8_t *)key, c) ? rn->r : rn->l;
@@ -834,13 +873,16 @@ sen_sym_prefix_search_with_set(sen_sym *sym, const void *key, sen_set *h)
     }
     if (!(k = pat_node_get_key(sym, rn))) { break; }
     if (!memcmp(k, key, len >> 3)) {
-      sen_set_get(h, &r, NULL);
-      if (rn->check >= len) { get_tc(sym, h, rn, (unsigned int)len); }
+      if (rn->check >= len) {
+        get_tc(sym, h, rn, (unsigned int)len);
+      } else {
+        sen_set_get(h, &r, NULL);
+      }
       return sen_success;
     }
     break;
   }
-  return sen_internal_error;
+  return sen_end_of_data;
 }
 
 sen_set *
@@ -897,8 +939,10 @@ sen_sym_common_prefix_search(sen_sym *sym, const void *key)
   if (!sym || !key || sym->key_size) { return SEN_SYM_NIL; } /* only for string */
   if (sym->v08p) { return sen_sym_common_prefix_search08(sym, key); }
   // len = (strlen(key) + 1) * 8;
-  for (r = pat_at(sym, 0)->r; r;) {
-    if (!(rn = pat_at(sym, r))) { break; /* corrupt? */ }
+  PAT_AT(sym, 0, rn);
+  for (r = rn->r; r;) {
+    PAT_AT(sym, r, rn);
+    if (!rn) { break; /* corrupt? */ }
     // if (len <= rn->check) { break; }
     if (rn->check <= c) {
       uint8_t *p = pat_node_get_key(sym, rn);
@@ -914,7 +958,8 @@ sen_sym_common_prefix_search(sen_sym *sym, const void *key)
       if (len2 <= c) {
         len2 = (n + 1) * 8;
         for (cl = c, rl = rn->l; rl; cl = rnl->check, rl = rnl->l) {
-          if (!(rnl = pat_at(sym, rl))) { break; /* corrupt? */ }
+          PAT_AT(sym, rl, rnl);
+          if (!rnl) { break; /* corrupt? */ }
           if (len2 <= rnl->check) { break; }
           if (rnl->check <= cl) {
             uint8_t *p = pat_node_get_key(sym, rnl);
@@ -946,12 +991,13 @@ _sen_sym_del(sen_sym *sym, const char *key, int shared)
   if (!len) { len = (strlen(key) + 1) * 8; }
   di = delinfo_new(sym); /* must be called before find rn */
   SET_DL_SH(di, shared ? DL_SHARED : 0);
-  rn = pat_at(sym, 0);
+  PAT_AT(sym, 0, rn);
   c = -1;
   p = &rn->r;
   for (;;) {
     if (!(r = *p)) { return sen_invalid_argument; }
-    if (!(rn = pat_at(sym, r))) { return sen_invalid_format; }
+    PAT_AT(sym, r, rn);
+    if (!rn) { return sen_invalid_format; }
     if (len <= rn->check) { return sen_invalid_argument; }
     if (c >= rn->check) {
       const uint8_t *k = pat_node_get_key(sym, rn);
@@ -970,7 +1016,7 @@ _sen_sym_del(sen_sym *sym, const char *key, int shared)
     SET_DL_ST(di, DL_PHASE2);
     di->d = r;
     if (otherside) {
-      rno = pat_at(sym, otherside);
+      PAT_AT(sym, otherside, rno);
       if (rno && c0 < rno->check && rno->check <= c) {
         if (!delinfo_search(sym, otherside)) {
           SEN_LOG(sen_log_error, "no delinfo found %d", otherside);
@@ -1032,7 +1078,7 @@ _sen_sym_del(sen_sym *sym, const char *key, int shared)
       rn0->check = 0;
     } else {
       if (otherside) {
-        rno = pat_at(sym, otherside);
+        PAT_AT(sym, otherside, rno);
         if (rno && c0 < rno->check && rno->check <= c) {
           if (!delinfo_search(sym, otherside)) {
             SEN_LOG(sen_log_error, "no delinfo found %d", otherside);
@@ -1058,7 +1104,8 @@ sen_sym_del(sen_sym *sym, const void *key)
   if (sym->flags & SEN_INDEX_SHARED_LEXICON) {
     sen_id id = sen_sym_at(sym, key);
     if (id) {
-      pat_node *rn = pat_at(sym, id);
+      pat_node *rn;
+      PAT_AT(sym, id, rn);
       if (rn && rn->bitfield >= 1 << 2) { return sen_success; }
     }
   }
@@ -1077,7 +1124,7 @@ _sen_sym_key(sen_sym *sym, sen_id id)
 {
   pat_node *node;
   if (sym->v08p) { return _sen_sym_key08(sym, id); }
-  node = pat_at(sym, id);
+  PAT_AT(sym, id, node);
   if (!node) { return NULL; }
   return (const char *) pat_node_get_key(sym, node);
 }
@@ -1090,7 +1137,8 @@ sen_sym_key(sen_sym *sym, sen_id id, void *keybuf, int bufsize)
   pat_node *node;
   if (!sym) { return sen_invalid_argument; }
   if (sym->v08p) { return sen_sym_key08(sym, id, keybuf, bufsize); }
-  if (!(node = pat_at(sym, id))) { return 0; }
+  PAT_AT(sym, id, node);
+  if (!node) { return 0; }
   if (!(key = pat_node_get_key(sym, node))) { return 0; }
   if (!(len = sym->key_size)) { len = (int)(strlen((char *)key) + 1); }
   if (keybuf && bufsize >= len) { memcpy(keybuf, key, len); }
@@ -1103,7 +1151,7 @@ sen_sym_pocket_get(sen_sym *sym, sen_id id)
   pat_node *node;
   if (!sym) { return -1; }
   if (sym->v08p) { return sen_sym_pocket_get08(sym, id); }
-  node = pat_at(sym, id);
+  PAT_AT(sym, id, node);
   if (!node) { return -1; }
   return PAT_PKT(node);
 }
@@ -1114,7 +1162,7 @@ sen_sym_pocket_set(sen_sym *sym, sen_id id, unsigned int value)
   pat_node *node;
   if (!sym) { return sen_invalid_argument; }
   if (sym->v08p) { return sen_sym_pocket_set08(sym, id, value); }
-  node = pat_at(sym, id);
+  PAT_AT(sym, id, node);
   if (value > PAT_POCKET_MAX || !node) { return sen_invalid_argument; }
   SET_PAT_PKT(node, value);
   return sen_success;
@@ -1126,7 +1174,7 @@ sen_sym_pocket_incr(sen_sym *sym, sen_id id)
   pat_node *node;
   if (!sym) { return sen_invalid_argument; }
   if (sym->v08p) { return sen_invalid_argument; }
-  node = pat_at(sym, id);
+  PAT_AT(sym, id, node);
   if (!node) { return sen_invalid_argument; }
   node->bitfield += (1 << 2);
   if (node->bitfield < (1 << 2)) {
@@ -1142,7 +1190,7 @@ sen_sym_pocket_decr(sen_sym *sym, sen_id id)
   pat_node *node;
   if (!sym) { return sen_invalid_argument; }
   if (sym->v08p) { return sen_invalid_argument; }
-  node = pat_at(sym, id);
+  PAT_AT(sym, id, node);
   if (!node) { return sen_invalid_argument; }
   if (node->bitfield < (1 << 2)) {
     SEN_LOG(sen_log_error, "sen_sym_pocket_decr failed %d", node->bitfield);
@@ -1183,7 +1231,8 @@ sen_sym_del_with_sis(sen_sym *sym, sen_id id,
   if (sym->v08p) { return sen_sym_del_with_sis08(sym, id, func, func_arg); }
   while (id) {
     if (sym->flags & SEN_INDEX_SHARED_LEXICON) {
-      pat_node *rn = pat_at(sym, id);
+      pat_node *rn;
+      PAT_AT(sym, id, rn);
       if (rn && rn->bitfield >= 1 << 2) { break; }
     }
     if ((si && si->children && si->children != id) || !func(id, func_arg)) { break; }
@@ -1226,7 +1275,8 @@ sen_sym_del_with_sis(sen_sym *sym, sen_id id,
     while (id && key) {
       if (_sen_sym_key(sym, id) != key) { break; }
       {
-        pat_node *rn = pat_at(sym, id);
+        pat_node *rn;
+        PAT_AT(sym, id, rn);
         if (!rn) { break; }
         if (lkey) {
           rn->key = lkey;
@@ -1283,7 +1333,7 @@ sen_sym_lock(sen_sym *sym, int timeout)
     }
     return sen_success;
   }
-  return sen_abnormal_error;
+  return sen_other_error;
 }
 
 sen_rc
@@ -1300,3 +1350,340 @@ sen_sym_clear_lock(sen_sym *sym)
   *sym->lock = 0;
   return sen_success;
 }
+
+int
+sen_sym_scan(sen_sym *sym, const char *str, unsigned int str_len,
+             sen_sym_scan_hit *sh, unsigned int sh_size, const char **rest)
+{
+  int n = 0;
+  sen_id tid;
+  if (sym->flags & SEN_INDEX_NORMALIZE) {
+    sen_nstr *nstr = sen_nstr_open(str, str_len, sym->encoding, SEN_STR_WITH_CHECKS);
+    if (nstr) {
+      int16_t *cp = nstr->checks;
+      unsigned int offset = 0, offset0 = 0;
+      const char *sp = nstr->norm, *se = nstr->norm + nstr->norm_blen;
+      while (n < sh_size) {
+        if ((tid = sen_sym_common_prefix_search(sym, sp))) {
+          int len = strlen(_sen_sym_key(sym, tid));
+          sh[n].id = tid;
+          sh[n].offset = (*cp > 0) ? offset : offset0;
+          while (len--) {
+            if (*cp > 0) { offset0 = offset; offset += *cp; }
+            sp++; cp++;
+          }
+          sh[n].length = offset - sh[n].offset;
+          n++;
+        } else {
+          if (*cp > 0) { offset0 = offset; offset += *cp; }
+          do {
+            sp++; cp++;
+          } while (sp < se && !*cp);
+        }
+        if (se <= sp) { offset = str_len; break; }
+      }
+      if (rest) { *rest = nstr->orig + offset; }
+      sen_nstr_close(nstr);
+    } else {
+      n = -1;
+      if (rest) { *rest = str; }
+    }
+  } else {
+    int len;
+    const char *sp, *se = str + str_len;
+    for (sp = str; sp < se && n < sh_size; sp += len) {
+      if ((tid = sen_sym_common_prefix_search(sym, sp))) {
+        len = strlen(_sen_sym_key(sym, tid));
+        sh[n].id = tid;
+        sh[n].offset = sp - str;
+        sh[n].length = len;
+        n++;
+      } else {
+        len = sen_str_charlen_nonnull(sp, se, sym->encoding);
+      }
+      if (!len) { break; }
+    }
+    if (rest) { *rest = sp; }
+  }
+  return n;
+}
+
+#define INITIAL_SIZE 512
+
+inline static void
+push(sen_sym_cursor *c, sen_id id, uint16_t check)
+{
+  sen_ctx *ctx = c->ctx;
+  sen_sym_cursor_entry *se;
+  if (c->size <= c->sp) {
+    if (c->ss) {
+      uint32_t size = c->size * 4;
+      sen_sym_cursor_entry *ss = SEN_REALLOC(c->ss, size);
+      if (!ss) { return; /* give up */ }
+      c->ss = ss;
+      c->size = size;
+    } else {
+      if (!(c->ss = SEN_MALLOC(sizeof(sen_sym_cursor_entry) * INITIAL_SIZE))) {
+        return; /* give up */
+      }
+      c->size = INITIAL_SIZE;
+    }
+  }
+  se = &c->ss[c->sp++];
+  se->id = id;
+  se->check = check;
+}
+
+inline static sen_sym_cursor_entry *
+pop(sen_sym_cursor *c)
+{
+  return c->sp ? &c->ss[--c->sp] : NULL;
+}
+
+sen_id
+sen_sym_cursor_next(sen_sym_cursor *c)
+{
+  pat_node *node;
+  sen_sym_cursor_entry *se;
+  while ((se = pop(c))) {
+    sen_id id = se->id;
+    uint16_t check = se->check;
+    for (;;) {
+      if (id) {
+        PAT_AT(c->sym, id, node);
+        if (node) {
+          if (node->check > check) {
+            if (c->flags & SEN_SYM_ASCENDING) {
+              push(c, node->r, node->check);
+              id = node->l;
+            } else {
+              push(c, node->l, node->check);
+              id = node->r;
+            }
+            check = node->check;
+            continue;
+          } else {
+            if (id == c->limit) { c->sp = 0; }
+            return id;
+          }
+        }
+      } else {
+        break;
+      }
+    }
+  }
+  return SEN_SYM_NIL;
+}
+
+void
+sen_sym_cursor_close(sen_sym_cursor *c)
+{
+  sen_ctx *ctx = c->ctx;
+  if (c->ss) { SEN_FREE(c->ss); }
+  SEN_FREE(c);
+}
+
+inline static int
+bitcmp(const void *s1, const void *s2, int offset, int length)
+{
+  int r, rest = length + (offset & 7) - 8, bl = offset >> 3, mask = 0xff >> (offset & 7);
+  unsigned char *a = (unsigned char *)s1 + bl, *b = (unsigned char *)s2 + bl;
+  if (rest <= 0) {
+    mask &= 0xff << -rest;
+    return (*a & mask) - (*b & mask);
+  }
+  if ((r = (*a & mask) - (*b & mask))) { return r; }
+  a++; b++;
+  if ((bl = rest >> 3)) {
+    if ((r = memcmp(a, b, bl))) { return r; }
+    a += bl; b += bl;
+  }
+  mask = 0xff << (8 - (rest & 7));
+  return (*a & mask) - (*b & mask);
+}
+
+inline static sen_rc
+set_cursor_ascend(sen_sym *sym, sen_sym_cursor *c, const void *key, int flags)
+{
+  sen_id id;
+  pat_node *node;
+  const uint8_t *k;
+  int r, check = -1, len, c2;
+  if (!(len = sym->key_size * 8)) { len = (strlen(key) + 1) * 8; }
+  PAT_AT(sym, 0, node);
+  for (id = node->r; id;) {
+    PAT_AT(sym, id, node);
+    if (!node) { return sen_invalid_format; }
+    if (node->check <= check) {
+      if (!(k = pat_node_get_key(sym, node))) { return sen_invalid_format; }
+      if (sym->key_size) {
+        if (flags & SEN_SYM_GT) {
+          if (memcmp(key, k, sym->key_size) < 0) { push(c, id, check); }
+        } else {
+          if (memcmp(key, k, sym->key_size) <= 0) { push(c, id, check); }
+        }
+      } else {
+        int l = (strlen(k) + 1) * 8;
+        if (l == len) {
+          if (flags & SEN_SYM_GT) {
+            if (memcmp(key, k, l >> 3) < 0) { push(c, id, check); }
+          } else {
+            if (memcmp(key, k, l >> 3) <= 0) { push(c, id, check); }
+          }
+        } else if (l < len) {
+          if (memcmp(key, k, l >> 3) < 0) { push(c, id, check); }
+        } else {
+          if (memcmp(key, k, len >> 3) <= 0) { push(c, id, check); }
+        }
+      }
+      break;
+    }
+    c2 = len < node->check ? len : node->check;
+    if (check + 1 < c2) {
+      if (!(k = pat_node_get_key(sym, node))) { return sen_invalid_format; }
+      if ((r = bitcmp(key, k, check + 1, c2 - (check + 1)))) {
+        if (r < 0) {
+          push(c, node->r, node->check);
+          push(c, node->l, node->check);
+        }
+        break;
+      }
+    }
+    if (len <= node->check) { break; }
+    check = node->check;
+    if (nth_bit((uint8_t *)key, check)) {
+      id = node->r;
+    } else {
+      push(c, node->r, node->check);
+      id = node->l;
+    }
+  }
+  return sen_success;
+}
+
+inline static sen_rc
+set_cursor_descend(sen_sym *sym, sen_sym_cursor *c, const void *key, int flags)
+{
+  sen_id id;
+  pat_node *node;
+  const uint8_t *k;
+  int r, check = -1, len, c2;
+  if (!(len = sym->key_size * 8)) { len = (strlen(key) + 1) * 8; }
+  PAT_AT(sym, 0, node);
+  for (id = node->r; id;) {
+    PAT_AT(sym, id, node);
+    if (!node) { return sen_invalid_format; }
+    if (node->check <= check) {
+      if (!(k = pat_node_get_key(sym, node))) { return sen_invalid_format; }
+      if (sym->key_size) {
+        if (flags & SEN_SYM_LT) {
+          if (memcmp(key, k, sym->key_size) > 0) { push(c, id, check); }
+        } else {
+          if (memcmp(key, k, sym->key_size) >= 0) { push(c, id, check); }
+        }
+      } else {
+        int l = (strlen(k) + 1) * 8;
+        if (l <= len) {
+          if ((flags & SEN_SYM_LT) && l == len) {
+            if (memcmp(key, k, l >> 3) > 0) { push(c, id, check); }
+          } else {
+            if (memcmp(key, k, l >> 3) >= 0) { push(c, id, check); }
+          }
+        } else {
+          if (memcmp(key, k, len >> 3) > 0) { push(c, id, check); }
+        }
+      }
+      break;
+    }
+    c2 = len < node->check ? len : node->check;
+    if (check + 1 < c2) {
+      if (!(k = pat_node_get_key(sym, node))) { return sen_invalid_format; }
+      if ((r = bitcmp(key, k, check + 1, c2 - (check + 1)))) {
+        if (r >= 0) {
+          push(c, node->l, node->check);
+          push(c, node->r, node->check);
+        }
+        break;
+      }
+    }
+    if (len <= node->check) {
+      push(c, node->l, node->check);
+      push(c, node->r, node->check);
+      break;
+    }
+    check = node->check;
+    if (nth_bit((uint8_t *)key, check)) {
+      push(c, node->l, node->check);
+      id = node->r;
+    } else {
+      id = node->l;
+    }
+  }
+  return sen_success;
+}
+
+sen_sym_cursor *
+sen_sym_cursor_open(sen_sym *sym, sen_ctx *ctx,
+                    const void *min, const void *max, int flags)
+{
+  sen_id id;
+  pat_node *node;
+  sen_sym_cursor *c;
+  if (!sym || !ctx) { return NULL; }
+  if (!(c = SEN_MALLOCN(sen_sym_cursor, 1))) { return NULL; }
+  c->sym = sym;
+  c->ctx = ctx;
+  c->size = 0;
+  c->sp = 0;
+  c->ss = NULL;
+  c->limit = 0;
+  if (flags & SEN_SYM_ASCENDING) {
+    if (max) {
+      set_cursor_descend(sym, c, max, flags);
+      c->flags = SEN_SYM_DESCENDING;
+      c->limit = sen_sym_cursor_next(c);
+      c->sp = 0;
+    }
+    if (min) {
+      set_cursor_ascend(sym, c, min, flags);
+    } else {
+      PAT_AT(sym, 0, node);
+      if (!node) {
+        sen_sym_cursor_close(c);
+        return NULL;
+      }
+      if ((id = node->r)) {
+        PAT_AT(sym, id, node);
+        if (node) {
+          push(c, node->r, node->check);
+          push(c, node->l, node->check);
+        }
+      }
+    }
+  } else {
+    if (min) {
+      set_cursor_ascend(sym, c, min, flags);
+      c->flags = SEN_SYM_ASCENDING;
+      c->limit = sen_sym_cursor_next(c);
+      c->sp = 0;
+    }
+    if (max) {
+      set_cursor_descend(sym, c, max, flags);
+    } else {
+      PAT_AT(sym, 0, node);
+      if (!node) {
+        sen_sym_cursor_close(c);
+        return NULL;
+      }
+      if ((id = node->r)) {
+        PAT_AT(sym, id, node);
+        if (node) {
+          push(c, node->l, node->check);
+          push(c, node->r, node->check);
+        }
+      }
+    }
+  }
+  c->flags = flags;
+  return c;
+}
