@@ -70,6 +70,23 @@ struct _sen_sym {
   void *sisaddrs[SEN_SYM_MAX_SEGMENT];
 };
 
+struct _sen_sym_cursor_entry {
+  sen_id id;
+  uint16_t check;
+};
+
+typedef struct _sen_sym_cursor_entry sen_sym_cursor_entry;
+
+struct _sen_sym_cursor {
+  sen_sym *sym;
+  sen_ctx *ctx;
+  unsigned int size;
+  unsigned int sp;
+  int flags;
+  sen_id limit;
+  sen_sym_cursor_entry *ss;
+};
+
 const char *_sen_sym_key(sen_sym *sym, sen_id id);
 // sen_id sen_sym_del_with_sis(sen_sym *sym, sen_id id);
 int sen_sym_del_with_sis(sen_sym *sym, sen_id id,
