@@ -262,7 +262,7 @@ check_inv(senchk_file *f)
 
     if (chkflags & CHK_UNLOCK) {
       printf("  unlock inv file: '%s'\n", f->path);
-      sen_index_clear_lock(i);
+      e = sen_index_clear_lock(i) == sen_success ? 0 : -5;
     } else {
       inv = i->inv;
       if (chkflags & CHK_DUMP_LEXICON) {
@@ -464,6 +464,7 @@ senchk_filelist_add(senchk_filelist *fl, senchk_filetype type,
       f->keyspath = f->lexpath = NULL;
     }
     f->path = strdup(path);
+    f->cerr = 0;
     fl->n_files++;
     senchk_filelist_expand(fl);
     return 1;
@@ -508,6 +509,7 @@ senchk_filelist_addi(senchk_filelist *fl, senchk_filetype type, void *instance)
         /* TODO: error */
         break;
     }
+    f->cerr = 0;
     fl->n_files++;
     senchk_filelist_expand(fl);
     return 1;
