@@ -46,7 +46,7 @@ sym_open(const char *filename)
 {
   sen_sym *sym;
   if (!(sym = sen_sym_open(filename))) {
-    sym = sen_sym_create(filename, 0, 0, sen_enc_euc_jp);
+    sym = sen_sym_create(filename, 0, SEN_INDEX_NORMALIZE, sen_enc_euc_jp);
   }
   return sym;
 }
@@ -68,11 +68,35 @@ do_insert(const char *filename, const char *string)
   sen_sym *sym;
   if (!(sym = sym_open(filename))) { return -1; }
   if (string) {
-    sen_sym_get(sym, string);
+    if (sym->flags & SEN_INDEX_NORMALIZE) {
+      sen_nstr *nstr;
+      if (!(nstr = sen_nstr_open(string, strlen(string), sym->encoding, 0))) {
+        return -1;
+      }
+      {
+        char *key = nstr->norm;
+        sen_sym_get(sym, chomp(key));
+      }
+      sen_nstr_close(nstr);
+    } else {
+      sen_sym_get(sym, string);
+    }
   } else {
     while (!feof(stdin)) {
       if (!fgets(keybuf, SEN_SYM_MAX_KEY_SIZE, stdin)) { break; }
-      sen_sym_get(sym, chomp(keybuf));
+      if (sym->flags & SEN_INDEX_NORMALIZE) {
+        sen_nstr *nstr;
+        if (!(nstr = sen_nstr_open(keybuf, strlen(keybuf), sym->encoding, 0))) {
+          return -1;
+        }
+        {
+          char *key = nstr->norm;
+          sen_sym_get(sym, chomp(key));
+        }
+        sen_nstr_close(nstr);
+      } else {
+        sen_sym_get(sym, chomp(keybuf));
+      }
     }
   }
   return 0;
@@ -97,21 +121,27 @@ do_delete(const char *filename, const char *string)
 inline static void
 doit(sen_sym *sym, const char *string)
 {
-  int len;
-  sen_id tid;
-  const char *p, *q, *r;
-  for (p = string; *p; p += len) {
-    if ((tid = sen_sym_common_prefix_search(sym, p))) {
-      len = sen_sym_key(sym, tid, keybuf, SEN_SYM_MAX_KEY_SIZE) - 1;
+  off_t off = 0;
+  const char *rest;
+  sen_sym_scan_hit sh[100];
+  size_t len = strlen(string);
+  while (off < len) {
+    int i, nhits = sen_sym_scan(sym, string + off, len - off, sh, 100, &rest);
+    for (i = 0, off = 0; i < nhits; i++) {
+      if (sh[i].offset < off) { continue; } /* skip overlapping region. */
+      fwrite(string + off, 1, sh[i].offset - off, stdout);
       fputs("<a href=\"http://d.hatena.ne.jp/keyword/", stdout);
+      sen_sym_key(sym, sh[i].id, keybuf, SEN_SYM_MAX_KEY_SIZE);
       encodeURL(keybuf);
       fputs("\">", stdout);
-      fputs(keybuf, stdout);
+      fwrite(string + sh[i].offset, 1, sh[i].length, stdout);
       fputs("</a>", stdout);
-    } else {
-      len = sen_str_charlen(p, sen_enc_euc_jp);
-      for (q = p, r = p + len; q < r; q++) { putchar(*q); }
+      off = sh[i].offset + sh[i].length;
     }
+    if (string + off < rest) {
+      fwrite(string + off, 1, rest - (string + off), stdout);
+    }
+    off = rest - string;
   }
 }
 
