@@ -300,15 +300,15 @@ def generate_map2(file, map2)
 end
 
 template = <<END
-/* Copyright(C) 2004 Brazil
+/* Copyright(C) 2004-2008 Brazil
 don't edit this file by hand. it generated automatically by nfkc.rb
 */
 
-#include "str.h"
+#include "nfkc.h"
 
 #ifndef NO_NFKC
 
-uint_least8_t
+unsigned char
 sen_nfkc_ctype(const unsigned char *str)
 {
 %  return -1;
